@@ -20,7 +20,16 @@
 
 #ifndef __BF_LIB_H_
 #define __BF_LIB_H_
+
+typedef struct {
+	gint my_int;
+	gchar *my_char;
+} Tconvert_table;
+
 gboolean file_copy(gchar *source, gchar *dest);
+gboolean append_to_file(gchar *filename, gchar *string);
+gint table_convert_char2int(Tconvert_table *table, gchar *my_char);
+gchar *table_convert_int2char(Tconvert_table *table, gint my_int);
 gchar *escapestring(gchar *original, gchar delimiter);
 gchar *unescapestring(gchar *original);
 gboolean change_dir(gchar * filename);
@@ -35,4 +44,8 @@ gchar *ending_slash(gchar *dirname);
 gint file_is_dir(gchar * filename);
 gint file_exists_and_readable(gchar * filename);
 gchar *bf_str_repeat(const gchar * str, gint number_of);
+gint get_int_from_string(gchar *string);
+gchar *create_secure_dir_return_filename();
+void remove_secure_dir_and_filename(gchar *filename);
+gchar *buf_replace_char(gchar *buf, gint len, gchar srcchar, gchar destchar);
 #endif /* __BF_LIB_H_ */
