@@ -18,6 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
 #include "default_include.h"
 
 #include <unistd.h> /* getuid */
@@ -25,9 +26,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/types.h>  /* msgsnd() */
-#include <locale.h>
+
+#ifdef HAVE_SYS_IPC_H  /* on win 32 we don't have it */
 #include <sys/ipc.h>  /* msgsnd() */
 #include <sys/msg.h>  /* msgsnd() */
+#endif
+
 #include <errno.h> 	/* errno */
 
 #if HAVE_LIBGDK_IMLIB
@@ -53,15 +57,15 @@ int main(gint argc, gchar * argv[]);
 void window_close(GtkWidget * button, gpointer windowname);
 void flush_queue(void);
 
-
-
 /****************************************************************************/
 
 void cleanup(void) {
 
+#ifdef HAVE_SYS_IPC_H
 #ifdef WITH_MSG_QUEUE
 	msgctl(main_v->msgid, IPC_RMID, 0);
 #endif /* WITH_MSG_QUEUE */
+#endif
 
 }
 
@@ -105,20 +109,32 @@ we just start, but we don't listen to the queue!
 
 gint msg_queue_open(void) {
 
+#ifdef HAVE_SYS_IPC_H
 	DEBUG_MSG("msg_queue_open, started\n");
 	main_v->msgid = msgget((key_t) BLUEFISH_MSG_QUEUE + getuid(), 0666 | IPC_CREAT | IPC_EXCL);
+#ifdef DEBUG
+		if (main_v->msgid == -1) {
+			DEBUG_MSG("msg_queue_open, errno=%d, error=%s\n", errno, g_strerror(errno));
+		}
+#endif
 	/* if main_v->msgid == -1 the message queue was already opened by another process */
 	DEBUG_MSG("msg_queue_open, main_v->msgid=%d\n", main_v->msgid);
 	if (main_v->msgid == -1) {
 		main_v->msgid = msgget((key_t) BLUEFISH_MSG_QUEUE+ getuid(), 0666);
-		DEBUG_MSG("msg_queue_open, errno=%d, error=%s\n", errno, g_strerror(errno));
+		DEBUG_MSG("msg_queue_open, connected to existing message queue, id=%d\n", main_v->msgid);
+#ifdef DEBUG
+		if (main_v->msgid == -1) {
+			DEBUG_MSG("msg_queue_open, errno=%d, error=%s\n", errno, g_strerror(errno));
+		}
+#endif
 		return 1;
-	} else {
-		return 0;
 	}
+#endif  
+		return 0;
 }
 
 gint msg_queue_check(gpointer data) {
+#ifdef HAVE_SYS_IPC_H
 	struct msgbuf {
 		long mtype;
 		char mtext[MSQ_QUEUE_SIZE];
@@ -128,15 +144,12 @@ gint msg_queue_check(gpointer data) {
 	if (main_v->msgid == -1) {
 		return FALSE;
 	}
-	DEBUG_MSG("msg_queue_check, checking\n");
+/*	DEBUG_MSG("msg_queue_check, checking\n"); */
 
 	retval = msgrcv(main_v->msgid, &msgp, MSQ_QUEUE_SIZE * sizeof(char), -MSG_QUEUE_OPENFILE, IPC_NOWAIT);
 	if (retval != -1) {
 		DEBUG_MSG("msg_queue_check, found type %ld\n", msgp.mtype);
-		if (msgp.mtype == MSG_QUEUE_OPENFILE) {
-			doc_new_with_file(msgp.mtext);
-			msg_queue_check(NULL); /* call it again, there may have been multiple files */
-		} else if (msgp.mtype == MSG_QUEUE_ASK_ALIVE) {
+		if (msgp.mtype == MSG_QUEUE_ASK_ALIVE) {
 			struct small_msgbuf {
 				long mtype;
 				char mtext[MSQ_QUEUE_SMALL_SIZE];
@@ -145,17 +158,25 @@ gint msg_queue_check(gpointer data) {
 			small_msgp.mtype = MSG_QUEUE_SEND_ALIVE;
 			strncpy(small_msgp.mtext,msgp.mtext, MSQ_QUEUE_SMALL_SIZE-1);	
 			msgsnd(main_v->msgid, (void *)&small_msgp, MSQ_QUEUE_SMALL_SIZE * sizeof(char), IPC_NOWAIT);
+		} else if (msgp.mtype == MSG_QUEUE_OPENFILE) {
+			DEBUG_MSG("msg_queue_check, a filename %s is received\n", msgp.mtext);
+			doc_new_with_file(msgp.mtext);
+			msg_queue_check(NULL); /* call it again, there may have been multiple files */
 		}
 	} else {
 		if (errno == 22) {
+			DEBUG_MSG("msg_queue_check, re-opening message queue ?!?!?\n");
 			/* the msg_queue was removed !?!?! */
 			msg_queue_open();
 		}
 	}
+#endif
 	return TRUE;
 }
 
 gint msg_queue_check_alive(gchar *pid_string) {
+
+#ifdef HAVE_SYS_IPC_H
 	struct small_msgbuf {
 		long mtype;
 		char mtext[MSQ_QUEUE_SMALL_SIZE];
@@ -171,7 +192,7 @@ gint msg_queue_check_alive(gchar *pid_string) {
 		}
 	}
 	DEBUG_MSG("msg_queue_check, errno=%d, error=%s\n", errno, g_strerror(errno));
-	DEBUG_MSG("msg_queue_check, no keepalive received, assuming server mode\n");
+#endif
 	return 0;
 }
 
@@ -180,11 +201,11 @@ gint msg_queue_check_alive(gchar *pid_string) {
 int main(gint argc, gchar * argv[])
 {
 #ifdef WITH_MSG_QUEUE
-	gint other_bluefish;
-	gint ask_other_alive;
+	gint other_bluefish=0, received_keepalive=0;
 	gchar *pid_string=NULL;
 #endif /* WITH_MSG_QUEUE */
 
+#ifdef ENABLE_NLS
 	/* init locale before any *_init */
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -192,6 +213,7 @@ int main(gint argc, gchar * argv[])
 
 	/* for XIM support we need to call gtk_set_locale() before gdk_init() */
 	gtk_set_locale();
+#endif
 	gdk_init(&argc, &argv);
 	gdk_rgb_init (); /* initialise GdkRgb */
 
@@ -205,6 +227,7 @@ int main(gint argc, gchar * argv[])
 	parse_commandline(argc, argv);
 
 	/* see if we're running as root, if so, exit with an error unless overriden */
+#ifndef WIN32
 	if (root_override != 1) {
 		if (geteuid() == 0) {
 			g_print(_("Error: cannot run as root (for security reasons)\n"));
@@ -217,6 +240,7 @@ int main(gint argc, gchar * argv[])
 			exit(EXIT_FAILURE);
 		}
 	}
+#endif
 
 	DEBUG_MSG("main, init main_v\n");
 	/*initialise the main struct */
@@ -227,70 +251,118 @@ int main(gint argc, gchar * argv[])
 	check_files_links_directories();
 	parse_bluefish_rcfile();
 
+#ifdef HAVE_SYS_IPC_H
 #ifdef WITH_MSG_QUEUE
-	DEBUG_MSG("main, open message queue\n");
+	DEBUG_MSG("main (msg queue), open message queue\n");
 	other_bluefish = msg_queue_open();
 	if (other_bluefish) {
 		struct small_msgbuf {
 			long mtype;
 			char mtext[MSQ_QUEUE_SMALL_SIZE];
 		} small_msgp;
+		gint ask_other_alive;
 		/* if the message queue is still open and the process listening is killed
 			we should be the server process --> we have to check this */
-		DEBUG_MSG("main, asking for keepalive\n");
 		pid_string = g_strdup_printf("%d", (int)getpid());
+		DEBUG_MSG("main, asking for keepalive, string %s\n", pid_string);
 		small_msgp.mtype = MSG_QUEUE_ASK_ALIVE;
 		strncpy(small_msgp.mtext, pid_string, MSQ_QUEUE_SMALL_SIZE-1);
 		ask_other_alive = msgsnd(main_v->msgid, (void *)&small_msgp, MSQ_QUEUE_SMALL_SIZE*sizeof(char), IPC_NOWAIT);
-
-	
+#ifdef DEBUG
+		if (ask_other_alive == -1) {
+			DEBUG_MSG("main (msg queue), errno=%d, error=%s\n", errno, g_strerror(errno));
+		}
+#endif
 		if (!load_projectname && load_filenames && main_v->props.open_in_running_bluefish) {
 			if (main_v->msgid != -1) {
 				struct msgbuf {
 					long mtype;
 					char mtext[MSQ_QUEUE_SIZE];
 				} msgp;
-				gint success=1;
+				gint success=1, check_keepalive_cnt=0, send_failure_cnt=0;
 				GList *tmplist;
 	
 				/* we have a message queue now, opened by another bluefish process */
 				msgp.mtype = MSG_QUEUE_OPENFILE;
 				tmplist = g_list_first(load_filenames);
-				while(tmplist) {
+				while(tmplist && success) {
 					gint retval;
 					gint len = strlen((gchar *)tmplist->data);
 					if (len < MSQ_QUEUE_SIZE-1) {
 						strncpy(msgp.mtext, (gchar *)tmplist->data, MSQ_QUEUE_SIZE-1);
 						retval = msgsnd(main_v->msgid, (void *)&msgp, MSQ_QUEUE_SIZE * sizeof(char), IPC_NOWAIT);
 						if (retval == -1) {
-							DEBUG_MSG("main, failed sending, errno=%d, error=%s\n", errno, g_strerror(errno));
-							success = 0;
+							DEBUG_MSG("main (msg queue), failed sending, errno=%d, error=%s\n", errno, g_strerror(errno));
+							usleep(MSG_QUEUE_PER_DOCUMENT_TIMEOUT * 1000);
+							if (received_keepalive) {
+								send_failure_cnt++;
+							} else {
+								if (msg_queue_check_alive(pid_string)) {
+									received_keepalive = 1;
+									DEBUG_MSG("main (msg queue), received keepalive\n");
+								} else {
+									check_keepalive_cnt++;
+									DEBUG_MSG("main (msg queue), no keepalive (try %d)\n", check_keepalive_cnt);
+								}
+							}
+						} else {
+							if (!received_keepalive) {
+								/* if we fill the message queue with loads of data, the server 
+								process doesn't even get a chance of reply-ing. So as long as we 
+								don't know a thing about it, we give it some time and check for
+								a reply often */
+								usleep(MSQ_QUEUE_CHECK_TIME * 1000);
+								if (msg_queue_check_alive(pid_string)) {
+									received_keepalive = 1;
+									DEBUG_MSG("main (msg queue), received keepalive\n");
+								} else {
+									check_keepalive_cnt++;
+									DEBUG_MSG("main (msg queue), no keepalive (try %d)\n", check_keepalive_cnt);
+								}
+							}
+							DEBUG_MSG("main (msg queue), sending %s succeeded\n", (gchar *)tmplist->data);
+							send_failure_cnt = 0;
+							tmplist = g_list_next(tmplist);
 						}
 					} else {
-						DEBUG_MSG("main, failed sending, length increased message size\n");
+						DEBUG_MSG("main (msg queue), failed sending, length increased message size\n");
+						success = 0;
+					}
+					if ((check_keepalive_cnt > 5) || (send_failure_cnt > 30)) {
+						DEBUG_MSG("main (msg queue), to many tries, check_keepalive_cnt=%d, send_failure_cnt=%d\n", check_keepalive_cnt, send_failure_cnt);
 						success = 0;
 					}
-					tmplist = g_list_next(tmplist);
 				}
 				if (success) {
-					DEBUG_MSG("main, sending filenames complete and successfull, ask_other_alive=%d\n",ask_other_alive);
+					DEBUG_MSG("main (msg queue), sending filenames complete and successfull, received_keepalive=%d\n",received_keepalive);
 					/* all filenames send to other process, test if it is alive */
-					if (ask_other_alive != -1) {
-						/* the other process should have enough time to check the queue */
-						/* the macro is in milliseconds, usleep is microseconds */
-						usleep( 1000 * MSQ_QUEUE_CHECK_TIME );
-						if (msg_queue_check_alive(pid_string)) {
-							/* we did receive a keep alive message! */
-							exit(0);
+					if (received_keepalive) {
+						exit(0);
+					} else {
+						if (ask_other_alive != -1) {
+							/* the other process should have enough time to check the queue */
+							/* the macro is in milliseconds, usleep is microseconds */
+							usleep( 1000 * MSQ_QUEUE_CHECK_TIME );
+							if (msg_queue_check_alive(pid_string)) {
+								/* we did receive a keep alive message! */
+								exit(0);
+							}
 						}
 					}
 				}
 			}
+		} else { /* if (!load_projectname && load_filenames && main_v->props.open_in_running_bluefish) */
+			usleep( 1000 * MSQ_QUEUE_CHECK_TIME);
+			if (msg_queue_check_alive(pid_string)) {
+				received_keepalive = 1;
+				DEBUG_MSG("main (msg queue), received keepalive, we should not be the server!\n");
+			}
 		}
-	}
+		
+	} /* if (other_bluefish) */
 	
 #endif /* WITH_MSG_QUEUE */
-
+#endif
 	/* display a splash screen with progress bar
 	 * also creates the main window, reads the config, 
 	 * checks directories etc... 
@@ -313,7 +385,8 @@ int main(gint argc, gchar * argv[])
 
 #ifdef WITH_MSG_QUEUE
 	/* there was a message queue, lets check if there is a server process reading it */
-	if (!other_bluefish || !msg_queue_check_alive(pid_string)) {
+	if (!other_bluefish || (other_bluefish && !received_keepalive)) {
+		DEBUG_MSG("main (msg queue), no other bluefish, or we didn't get a keepalive, we will be server!\n");
 		gtk_timeout_add(MSQ_QUEUE_CHECK_TIME, msg_queue_check, NULL);
 	}
 	if (pid_string) {
@@ -345,7 +418,11 @@ void load_identify_filters() {
 	gchar * jscript[] = {".js", NULL};
 	gchar * css[] = {".css", NULL};
 	gchar * other[] = {".htaccess", ".asp", ".pl", ".cgi", ".jsp", NULL};
+#ifdef WIN32
+	gchar * dir[] = {"\\", NULL};
+#else
 	gchar * dir[] = {"/", NULL};
+#endif
 	gint count;
 
 	for (count = 0 ; count < TYPES_COUNT ; count++) {
