@@ -2,6 +2,7 @@
  * bluefish.h - global prototypes live here
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
+ * Copyright (C) 1999-2001 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,16 +18,30 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+#define UNDO2
+/* #define DEVELOPMENT */
+/* #define IMAGEMAP_EDITOR_TEST */
+
 
 #ifndef __BLUEFISH_H_
 #define __BLUEFISH_H_
 
 #include <glib.h>
 #include <gtk/gtkwidget.h>
+#include <sys/types.h> /* time_t */
+
+#ifdef USE_GDK_PIXBUF
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#endif
 
 /* needed to see if WITH_SPC is defined */
 #include "config.h"
 
+#if defined(AUTOCOMPLET) || defined(ATTRPAGE)
+#define PARSEDTD
+#endif
+
+
 #define WITH_MSG_QUEUE
 
 #ifdef WITH_MSG_QUEUE
@@ -37,9 +52,30 @@
 #define MSG_QUEUE_SEND_ALIVE 46064
 #define MSG_QUEUE_OPENFILE 46063
 #define MSG_QUEUE_ASK_ALIVE 46062
-
+#define MSG_QUEUE_PER_DOCUMENT_TIMEOUT 150
 #endif
 
+
+
+#ifdef UNDO2
+typedef enum {
+	UndoDelete = 1, UndoInsert
+} undo_op_t;
+
+typedef struct {
+	GList *entries;	/* the list of entries that should be undone in one action */
+	gint changed;		/* doc changed status at this undo node */
+} unregroup_t;
+
+typedef struct {
+	GList *first;
+	GList *last;
+	unregroup_t *current;
+	gint num_groups;
+	GList *redofirst;
+} unre_t;
+#endif /* UNDO2 */
+
 /* the saved selection in a document */
 typedef struct {
 	gboolean saved;
@@ -52,17 +88,30 @@ typedef struct {
 typedef struct {
 	gchar *filename;
 	gint modified;
-	unsigned del_txt_id;		/* Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com> */
-	unsigned ins_txt_id;		/* undo/redo part is copied from gnotepad code */
-	GList *undolist;
-	GList *undotail;
+	time_t mtime; /* from stat() */
+	uid_t owner_uid; /* user ID of owner */
+	gid_t owner_gid; /* group ID of owner */
+	mode_t mode; /* mode field in stat() format so any stat macro's will work*/
+	gint is_symlink; /* file is a symbolic link */
+	unsigned del_txt_id;
+	unsigned ins_txt_id;
+#ifdef UNDO2
+	unre_t unre;
+#else
+	GList *undolist; /* Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com> */
+	GList *undotail; /* undo/redo part is copied from gnotepad code */
 	GList *redolist;
 	GList *redotail;
+#endif /* UNDO2 */
 	GtkWidget *textbox;
 	GtkWidget *tab_label;
 	gint highlightstate;
 	gint need_highlighting;		/* +1 means a '"' is added, +2 means a '<' is added, 4 means a '>' is added */
 	TSelection saved_sel;
+#ifdef PARSEDTD
+	int dtd_doctype; /* index of the doc's doctype in the array HTML_doctypes */
+	int dtd_loaded;  /* loaded doctype (may not be the same as above) */
+#endif
 } Tdocument;
 
 /* This is the current project */
@@ -86,6 +135,7 @@ typedef struct {
 	gint v_html_tb;				/* view html toolbar */
 	gint v_custom_tb;			/* view custom menubar */
 	gint v_main_tb;				/* view main toolbar */
+	gint v_filebrowser;  	/* view filebrowser */
 	gint transient_htdialogs;  /* set html dialogs transient ro the main window */
 	gint main_window_h;			/* main window height */
 	gint main_window_w;			/* main window width */
@@ -100,6 +150,7 @@ typedef struct {
 	gchar *filebrowser_filter_css; /* the css type */
 	gchar *filebrowser_filter_other; /* the other type */
 	gchar *cfg_browser_cline;	/* browser commandline */
+	gchar *image_editor_cline; 	/* image editor commandline */
 	gchar *cfg_weblint_cline;	/* weblint command line */
 	gchar *cfg_editor_font;		/* editor font */
 	gint cfg_editor_tabwidth;	/* editor tabwidth */
@@ -152,8 +203,12 @@ typedef struct {
 #endif
 	gchar *backup_filestring;  /* the string to append to the backup file */
 	gint backup_file; 			/* wheather to use a backup file */
+	gint backup_by_copy; 	/* make a copy instead of renaming the file */
+	gchar *backup_abort_style; /* if the backup fails, continue 'save', 'abort' save, or 'ask' user */
+	gint auto_convert_CR; /* auto remove carriage returns from input files */
 	gint allow_multi_instances; /* allow multiple instances of the same file */
-
+	gint num_undo_levels; 	/* number of undo levels per document */
+	gint clear_undo_on_save; 	/* clear all undo information on file save */
 	/* key conversion */
 	gint conv_ctrl_enter;		/* convert control-enter key press */
 	gchar *ctrl_enter_text;		/* inserted text */
@@ -163,8 +218,17 @@ typedef struct {
 #ifdef WITH_MSG_QUEUE
 	gint open_in_running_bluefish; /* open commandline documents in already running session*/
 #endif /* WITH_MSG_QUEUE */
+#ifdef AUTOCOMPLET
+	gint autocomplet_html;   /* Shows autocompletion window when editing HTML */
+	gint autocomplet_delay;  /* Delay to show the autocompletion window */
+#endif
 } Tproperties;
 
+typedef struct {
+	gchar *before;
+	gchar *after;
+} Tlast_insert;
+
 #ifdef WITH_INTERNAL_PREVIEW
 typedef struct {
 	GtkWidget *win;
@@ -181,6 +245,7 @@ FILTER_DOCUMENTS
 
 
 typedef struct {
+	GtkWidget *hpane; /* it needs to be in here so it can be destroyed when the filebrowser is removed */
 	gchar *curdir;
 	GtkWidget *dir_history;
 	GtkWidget *dir_list;
@@ -193,6 +258,20 @@ typedef struct {
 	GList *directory_history;
 } Tfilebrowser;
 
+typedef struct {
+     GtkWidget *w_img_preview_win;
+     GtkWidget *w_image;
+     gint max_width; /* make this configurable */
+     gint max_height; 
+     GtkWidget *w_scrolledwin;
+     char *current_img_filename;
+} Timage_preview;
+
+typedef struct {
+	GtkWidget *undo;
+	GtkWidget *redo;
+} Ttoolbar_buttons;
+
 /* identifying files */
 #define TYPE_UNKNOWN 0
 #define TYPE_IMAGE 1
@@ -205,31 +284,38 @@ typedef struct {
 #define TYPES_COUNT 8
 
 typedef struct {
-	Tproject current_project;
-	Tproperties props;
-	Tdocument *current_document;	/* This is always the current visible one */
-	GList *documentlist;		/* The documents are now placed in a Glist */
-	GtkWidget *notebook;
-	gint last_notebook_page;	/* this is the last used, this can be used to track down if the notebook page really changed */
-	GtkWidget *statusbar;
-	GtkWidget *menubar;
-	Tfilebrowser filebrowser;
-	gchar **identify_filters[TYPES_COUNT];
-	GtkWidget *main_window;
-	GtkWidget *cmenu;
-	GList *cmenu_arrays; /* custom menu strctures */
+     Tproject current_project;
+     Tproperties props;
+     Tdocument *current_document;	/* This is always the current visible one */
+     GList *documentlist;		/* The documents are now placed in a Glist */
+     GtkWidget *notebook;
+     gint last_notebook_page;	/* this is the last used, this can be used to track down if the notebook page really changed */
+     GtkWidget *statusbar;
+     GtkWidget *menubar;
+     Tfilebrowser filebrowser;
+     gchar **identify_filters[TYPES_COUNT];
+     GtkWidget *main_window;
+	  GtkWidget *middlebox; /* the hbox which contains the notebook or the notebook and the filebrowser */
+     GtkWidget *cmenu;
+     GList *cmenu_arrays; /* custom menu strctures */
 #ifdef WITH_INTERNAL_PREVIEW
-	Tpreview *preview;
+     Tpreview *preview;
 #endif
-	void *syntax_structs;		/* pointer to an array of struct : the props.syntax_configstrings parsed into structs with GdkColor and regex pattern */
-	gint syntax_structs_n;		/* number of structs pointed by syntax_structs */
-	GList *command_strings;
-	GList *filter_strings;
-	GList *recent_files;
-	GdkColormap *colormap;
+     void *syntax_structs;		/* pointer to an array of struct : the props.syntax_configstrings parsed into structs with GdkColor and regex pattern */
+     gint syntax_structs_n;		/* number of structs pointed by syntax_structs */
+     GList *command_strings;
+     GList *filter_strings;
+     GList *recent_files;
+     GdkColormap *colormap;
 #ifdef WITH_MSG_QUEUE
-	gint msgid;
+     gint msgid;
 #endif /* WITH_MSG_QUEUE */
+     Tlast_insert last_insert;
+     Timage_preview *img_preview;
+	  Ttoolbar_buttons toolb;
+#ifdef ATTRPAGE
+     GtkWidget *attrpage_window;
+#endif
 } Tmain;
 
 extern Tmain *main_v;
