@@ -19,8 +19,6 @@
  */
 #include "default_include.h"
 
-#include <locale.h>
-
 #include <stdlib.h>				/* system() */
 #include <gdk/gdkkeysyms.h>		/* GDK_Escape */
 #include <stdio.h>				/* FILE */
@@ -58,6 +56,26 @@ typedef struct {
 
 /****************************************************************/
 
+void set_last_insert(const gchar *before, const gchar *after) {
+
+	if (main_v->last_insert.before) {
+		main_v->last_insert.before = NULL;
+	}
+	if (main_v->last_insert.after) {
+		main_v->last_insert.after = NULL;
+	}
+	if (before && strlen(before)) {
+		main_v->last_insert.before = g_strdup(before);
+	}
+	if (after && strlen(after)) {
+		main_v->last_insert.after = g_strdup(after);
+	}
+}
+
+void repeat_last_insert_cb(GtkWidget * widget, gpointer data) {
+	insert_dbl_text(main_v->last_insert.before, main_v->last_insert.after);
+}
+
 /* view in netscape functions */
 
 void view_in_netscape_cb(GtkWidget * widget, gpointer data)
@@ -113,6 +131,15 @@ void view_in_netscape_cb(GtkWidget * widget, gpointer data)
 	g_free(url2);
 }
 
+void open_image_in_editor(gchar *filename) {
+	gchar *command;
+	gint rcode;
+	command = g_strdup_printf(main_v->props.image_editor_cline, filename);
+	rcode = system(command);
+	DEBUG_MSG("open_image_in_editor, gimp-remote returned %d\n");
+	g_free(command);
+}
+
 static void key_press_lcb(GtkWidget * widget, GdkEventKey * event, gpointer data)
 {
 	DEBUG_MSG("key_press_callback, Value is .%d.\n", event->keyval);
@@ -254,17 +281,32 @@ void run_weblint_cb(GtkWidget * widget, gpointer data)
 	GtkWidget *ext_window, *list, *close_button, *recheck_button;
 	GtkWidget *vbox, *hbox;
 	GtkWidget *scrolwin;
-	gchar *temp, *html_file, *output_file, *command;
+	gchar *temp, *output_file, *command;
 	gchar *text[1];
 	FILE *fd;
 	struct stat st_output_file;
+	
+	if (main_v->current_document->modified || !main_v->current_document->filename) {
+		/* ask if we should save */
+		gchar *options[] = {N_("Save"), N_("Abort"), NULL};
+		gchar *message;
+		gint retval;
+		if (main_v->current_document->filename) {
+			message = g_strconcat(_("File "), main_v->current_document->filename, _("\nis modified"), NULL);
+		} else {
+			message = g_strconcat(_("File has not been saved yet"), NULL);
+		}
+		retval = multi_button_dialog(_("Bluefish: Weblint warning"), 0, message, options);
+		g_free(message);
+		if (retval == 0) {
+			doc_save(main_v->current_document, 0, 0);
+		} 
+	}
+	if (main_v->current_document->modified || !main_v->current_document->filename) {
+		return;
+	}
 
-	output_file = g_strdup_printf("%s", "weblint.out");
-
-/* Current HTML file saved to filename 'untitled' */
-
-	html_file = g_strdup_printf("%s", "untitled");
-
+	output_file = create_secure_dir_return_filename();
 	statusbar_message(_("Running Weblint"), 1000);
 
 /* Main Window */
@@ -322,8 +364,7 @@ void run_weblint_cb(GtkWidget * widget, gpointer data)
 
 	/* Exec Weblint */
 
-	doc_textbox_to_file(main_v->current_document, html_file);
-	temp = g_strdup_printf("%s %s > %s 2>/dev/null", command, html_file, output_file);
+	temp = g_strdup_printf("%s %s > %s 2>/dev/null", command, main_v->current_document->filename, output_file);
 	flush_queue();
 
 	DEBUG_MSG("Weblint command: %s\n", temp);
@@ -373,14 +414,9 @@ void run_weblint_cb(GtkWidget * widget, gpointer data)
 		gtk_widget_show(ext_window);
 	}
 
-	code = unlink(html_file);
-	if (code == -1) {
-		DEBUG_MSG("Cannot delete file %s\n", html_file);
-		error_dialog(_("Weblint"), _("Error delete temporary saved file"));
-	}
+	remove_secure_dir_and_filename(output_file);
 
 	g_free(temp);
-	g_free(html_file);
 	g_free(output_file);
 }
 
@@ -437,3 +473,136 @@ void preview_cb(GtkWidget * widget, gpointer data) {
 }
 #endif
 
+typedef struct {
+	GList *filenames_to_return;
+	GtkWidget *win;
+	GtkWidget *basedir;
+	GtkWidget *find_pattern;
+	GtkWidget *recursive;
+	GtkWidget *grep_pattern;
+	GtkWidget *is_regex;
+} Tfiles_advanced;
+
+static void files_advanced_win_destroy(GtkWidget * widget, GdkEvent *event,  Tfiles_advanced *tfs) {
+	gtk_main_quit();
+	window_destroy(tfs->win);
+}
+
+static void files_advanced_win_ok_clicked(GtkWidget * widget, Tfiles_advanced *tfs) {
+	/* create list here */
+	gchar *command, *temp_file;
+	gchar *c_basedir, *c_find_pattern, *c_recursive, *c_grep_pattern, *c_is_regex;
+	gint statusid;
+	temp_file = create_secure_dir_return_filename();
+	if (!temp_file) {
+		files_advanced_win_destroy(widget, NULL, tfs);
+		DEBUG_MSG("files_advanced_win_ok_clicked, can't get a secure temp filename ?????\n");
+		return;
+	}
+	DEBUG_MSG("files_advanced_win_ok_clicked, temp_file=%s\n", temp_file);
+	c_basedir = gtk_editable_get_chars(GTK_EDITABLE(tfs->basedir), 0, -1);
+	c_find_pattern = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(tfs->find_pattern)->entry), 0, -1);	
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->recursive))) {
+		c_recursive= "-maxdepth 100 ";
+	} else {
+		c_recursive = "-maxdepth 1 ";
+	}
+
+	c_grep_pattern = gtk_editable_get_chars(GTK_EDITABLE(tfs->grep_pattern), 0, -1);
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tfs->is_regex))) {
+		c_is_regex = "-E -l ";
+	} else {
+		c_is_regex = "-l ";
+	}
+/*
+command = `find c_basedir -name c_find_pattern c_recursive`
+command = `grep -E 'c_grep_pattern' `find c_basedir -name c_find_pattern c_recursive``
+*/
+	if (strlen(c_grep_pattern) == 0) {
+		command = g_strconcat(PATH_TO_FIND, " ", c_basedir, " -name '", c_find_pattern, "' ", c_recursive, "> ", temp_file, NULL);
+	} else {
+		command = g_strconcat(PATH_TO_GREP, " ",c_is_regex, " '", c_grep_pattern,"' `", PATH_TO_FIND, " ", c_basedir, " -name '", c_find_pattern, "' ", c_recursive, "`", "> ", temp_file, NULL);
+	}
+	g_free(c_basedir);
+	g_free(c_find_pattern);
+	g_free(c_grep_pattern);
+	DEBUG_MSG("files_advanced_win_ok_clicked, command=%s\n", command);
+	statusid = bf_statusbar_message(command);
+	flush_queue();
+	system(command);
+	statusbar_remove(statusid);
+	tfs->filenames_to_return = get_stringlist(temp_file, tfs->filenames_to_return);
+	g_free(command);
+	remove_secure_dir_and_filename(temp_file);
+	g_free(temp_file);
+	files_advanced_win_destroy(widget, NULL, tfs);
+}
+static void files_advanced_win_cancel_clicked(GtkWidget * widget, Tfiles_advanced *tfs) {
+	files_advanced_win_destroy(widget, NULL, tfs);
+}
+
+static void files_advanced_win(Tfiles_advanced *tfs) {
+	GtkWidget *vbox, *hbox, *frame, *vbox2, *but, *hbox2;
+	GList *list;
+	gchar *curdir=g_get_current_dir();
+	tfs->win = window_full(_("Advanced file selector"), GTK_WIN_POS_CENTER, GTK_WINDOW_DIALOG, 5,
+			files_advanced_win_destroy, tfs);
+	tfs->filenames_to_return = NULL;
+	vbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(tfs->win), vbox);
+	frame = gtk_frame_new(_("Filename"));
+	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 5);
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(frame), vbox2);
+	/* filename part */
+	/* curdir should get a value */
+	tfs->basedir = boxed_full_entry(_("Basedir"), curdir, 255, vbox2);
+	g_free(curdir);
+	hbox2 = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox2), hbox2, TRUE, TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(hbox2), gtk_label_new(_("Pattern")), TRUE, TRUE, 5);
+
+	list = g_list_append(NULL, "*.php");
+	list = g_list_append(list, "*.php3");
+	list = g_list_append(list, "*.html");
+	list = g_list_append(list, "*.htm");
+	list = g_list_append(list, "*.shtml");
+	list = g_list_append(list, "*.pl");
+	list = g_list_append(list, "*.cgi");
+	list = g_list_append(list, "*.xml");
+	tfs->find_pattern = combo_with_popdown_sized("", list, 1, 300);
+	gtk_box_pack_start(GTK_BOX(hbox2), tfs->find_pattern, TRUE, TRUE, 5);
+	g_list_free(list);
+	tfs->recursive = boxed_checkbut_with_value(_("recursive"), 1, vbox2);
+
+	frame = gtk_frame_new(_("Contains"));
+	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 5);
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(frame), vbox2);
+	/* content */
+	tfs->grep_pattern = boxed_full_entry(_("Pattern"), NULL, 255, vbox2);
+	tfs->is_regex = boxed_checkbut_with_value(_("is regex"), 0, vbox2);
+	
+	/* buttons */
+	hbox = gtk_hbutton_box_new();
+	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 1);
+	but = bf_stock_ok_button(files_advanced_win_ok_clicked, tfs);
+	gtk_box_pack_start(GTK_BOX(hbox),but , FALSE, FALSE, 0);
+	gtk_window_set_default(GTK_WINDOW(tfs->win), but);
+	but = bf_stock_cancel_button(files_advanced_win_cancel_clicked, tfs);
+	gtk_box_pack_start(GTK_BOX(hbox),but , FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show_all(GTK_WIDGET(tfs->win));
+	gtk_grab_add(GTK_WIDGET(tfs->win));
+	gtk_widget_realize(GTK_WIDGET(tfs->win));
+	gtk_window_set_transient_for(GTK_WINDOW(tfs->win), GTK_WINDOW(main_v->main_window));
+}
+
+GList *return_files_advanced() {
+	Tfiles_advanced tfs = {NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+
+	files_advanced_win(&tfs);
+	gtk_main();
+	return tfs.filenames_to_return;
+}
