@@ -23,9 +23,13 @@
 
 #include <glib.h>
 #include <gtk/gtkwidget.h>
+void set_last_insert(const gchar *before, const gchar *after);
+void repeat_last_insert_cb(GtkWidget * widget, gpointer data);
 
 void view_in_netscape_cb(GtkWidget * widget, gpointer data);
+void open_image_in_editor(gchar *filename);
 void open_from_web_cb(GtkWidget * widget, gpointer data);
 void run_weblint_cb(GtkWidget * widget, gpointer data);
 void preview_cb(GtkWidget * widget, gpointer data);
+GList *return_files_advanced();
 #endif							/* __CLALLBACKS_H_ */
