@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * configure_cb.c
  *
- * Copyright (C) 1999 Olivier Sessink
+ * Copyright (C) 1999-2001 Olivier Sessink
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #include "default_include.h"
 
 #include <string.h>
-#include <locale.h>
 #include <ctype.h> 	/* isspace() */
 
 #include "bluefish.h"
@@ -31,20 +30,25 @@
 #include "interface.h"
 #include "highlight.h"
 #include "document.h"			/* set fonts */
+#include "config.h"
 
-typedef enum { none, file, font } Textra_but;
 
-static GtkWidget *font_e, *browser_e, *weblint_e, *tabfont_e, *tabpos_c;
+static GtkWidget *font_e, *browser_e, *weblint_e, *tabfont_e, *tabpos_c, *image_editor_cline_e;
 static GtkWidget *width_s, *height_s, *tab_s, *thumbtype_c, *thumbstring_e, *max_recent_s, *max_dir_his_s, *filelist_filter_c;
 static GtkWidget *full_li_c, *full_p_c, *lowercase_tags_c, *word_wrap_c, *autoindent_c , *line_wrap_c, *fontset_c, *link_management_c, *defaulthighlight_c, *conv_shift_enter_c, *conv_ctrl_enter_c, *shift_enter_text_c, *ctrl_enter_text_c, *conv_special_char_c;
 static GtkWidget *cont_high_up_c, *cont_highlight_full_c, *allow_css_c, *html_ver_c, *allow_dep_c, *xhtml_c, *allow_ruby_c, *allow_frames_c, *force_dtd_c, *dtd_url_c, *allow_h4_c, *xml_start_c;
-static GtkWidget *backup_filestring_e, *backup_file_c, *allow_multi_instances_c;
+static GtkWidget *backup_filestring_e, *backup_file_c, *allow_multi_instances_c,
+						*backup_by_copy_c, *backup_abort_style_c;
 static GtkWidget *file_width_s, *force_def_style_c, *drop_at_drop_pos_c;
 static GtkWidget *filebrowser_filter_image_e, *filebrowser_filter_php_e, *filebrowser_filter_html_e, *filebrowser_filter_jscript_e, *filebrowser_filter_css_e, *filebrowser_filter_other_e;
 static GtkWidget *format_by_context_c, *transient_htdialogs_c;
+static GtkWidget *num_undo_levels_s,*clear_undo_on_save_c;
 #ifdef WITH_MSG_QUEUE
 static GtkWidget *open_in_running_bluefish_c;
 #endif
+#ifdef AUTOCOMPLET
+static GtkWidget *autocomplet_html_c, *autocomplet_delay_s;
+#endif
 #ifdef WITH_SPC
 static GtkWidget *spc_command_e, *spc_lang_e, *spc_acc_comp_c, *spc_use_esc_c,
           *spc_use_pers_dict_c, *spc_pers_dict_e, *spc_use_input_encoding_c,
@@ -71,7 +75,7 @@ static void check_apply(gint * config_var, GtkWidget * check_button);
  *
  *************************/
 
-static GtkWidget *string_entry2(gchar * labeltext, gchar * which_config_string, GtkWidget * box, Textra_but with_but)
+static GtkWidget *string_entry2(gchar * labeltext, gchar * which_config_string, GtkWidget * box, Textra_but with_but, GtkWidget *win)
 {
 	GtkWidget *hbox, *return_widget, *font_b=NULL;
 
@@ -80,7 +84,7 @@ static GtkWidget *string_entry2(gchar * labeltext, gchar * which_config_string,
 	return_widget = boxed_entry_with_text(which_config_string, 1023, hbox);
 	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 3);
 	if (with_but == file) {
-		gtk_box_pack_start(GTK_BOX(hbox), file_but_new(return_widget, gtk_widget_get_toplevel(box), 1), FALSE, FALSE, 3);
+		gtk_box_pack_start(GTK_BOX(hbox), file_but_new(return_widget, win, 1), FALSE, FALSE, 3);
 	} else if (with_but == font) {
 		font_b = gtk_button_new_with_label(_("Font..."));
 		gtk_signal_connect(GTK_OBJECT(font_b), "clicked", GTK_SIGNAL_FUNC(choose_font), return_widget);
@@ -216,7 +220,9 @@ static void save_n_apply_config()
 	GList *tmplist;
 	Tdocument *tmpdoc;
 
+	DEBUG_MSG("apply_config, saving config\n");
 	save_config_files();
+	DEBUG_MSG("apply_config, applying config\n");
 	
 	/* actually try to apply these config values */
 	gtk_window_set_policy(GTK_WINDOW(main_v->main_window), FALSE, FALSE, FALSE);
@@ -260,6 +266,7 @@ static void unescape_string_apply(gchar ** config_var, GtkWidget * entry) {
 static void configure_ok_lcb(GtkWidget * button, gpointer entry) {
 	statusbar_message(_("Saving and applying configuration"), 1000);
 	string_apply(&main_v->props.cfg_browser_cline, browser_e);
+	string_apply(&main_v->props.image_editor_cline, image_editor_cline_e);
 	string_apply(&main_v->props.cfg_weblint_cline, weblint_e);
 	string_apply(&main_v->props.cfg_editor_font, font_e);
 	string_apply(&main_v->props.cfg_tab_font, tabfont_e);
@@ -300,6 +307,8 @@ static void configure_ok_lcb(GtkWidget * button, gpointer entry) {
 	check_apply(&main_v->props.drop_at_drop_pos, drop_at_drop_pos_c);
 	check_apply(&main_v->props.link_management, link_management_c);
 	check_apply(&main_v->props.allow_multi_instances, allow_multi_instances_c);
+	integer_apply(&main_v->props.num_undo_levels, num_undo_levels_s);
+	check_apply(&main_v->props.clear_undo_on_save, clear_undo_on_save_c);
 	check_apply(&main_v->props.defaulthighlight, defaulthighlight_c);
 	check_apply(&main_v->props.lowercase_tags, lowercase_tags_c);
 	check_apply(&main_v->props.cont_highlight_update, cont_high_up_c);
@@ -318,8 +327,14 @@ static void configure_ok_lcb(GtkWidget * button, gpointer entry) {
 #endif
 	string_apply(&main_v->props.backup_filestring, backup_filestring_e);
    check_apply(&main_v->props.backup_file, backup_file_c);
+	check_apply(&main_v->props.backup_by_copy, backup_by_copy_c);
+	string_apply(&main_v->props.backup_abort_style, GTK_COMBO(backup_abort_style_c)->entry);
 #ifdef WITH_MSG_QUEUE
 	check_apply(&main_v->props.open_in_running_bluefish, open_in_running_bluefish_c);
+#endif
+#ifdef AUTOCOMPLET
+	check_apply(&main_v->props.autocomplet_html, autocomplet_html_c);
+	integer_apply(&main_v->props.autocomplet_delay, autocomplet_delay_s);
 #endif
 	check_apply(&main_v->props.conv_shift_enter, conv_shift_enter_c);
 	check_apply(&main_v->props.conv_ctrl_enter, conv_ctrl_enter_c);
@@ -351,8 +366,8 @@ void configure_cb(GtkWidget * widget, gpointer page)
 
 	/* create the widgets */
 	DEBUG_MSG("configure_cb, started\n");
-	count = bf_statusbar_message(_("Configuration Window"));
-	dialog = window_with_title(_("Bluefish: configure"), GTK_WIN_POS_NONE, GTK_WINDOW_DIALOG, 5);
+	count = bf_statusbar_message(_("Edit preferences"));
+	dialog = window_with_title(_("Bluefish: Preferences"), GTK_WIN_POS_NONE, GTK_WINDOW_DIALOG, 5);
 	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 	gtk_signal_connect(GTK_OBJECT(dialog), "destroy", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 
@@ -371,7 +386,7 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
-	font_e = string_entry2(_("Default font"), main_v->props.cfg_editor_font, vbox, font);
+	font_e = string_entry2(_("Default font"), main_v->props.cfg_editor_font, vbox, font, dialog);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
@@ -390,6 +405,8 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	cont_high_up_c = check_entry2(_("Update syntax highlighting continuous"), main_v->props.cont_highlight_update, vbox1);
 	cont_highlight_full_c = check_entry2(_("Update full widget instead of the current line (slower)"), main_v->props.cont_highlight_full, vbox1);
 	link_management_c = check_entry2(_("Automatic link management"), main_v->props.link_management, vbox1);
+	num_undo_levels_s = spin_button2(_("Number of undo levels"), main_v->props.num_undo_levels, vbox1, 10.0, 10000.0);	
+	clear_undo_on_save_c = check_entry2(_("Clear undo levels on save"), main_v->props.clear_undo_on_save, vbox1);
 
 	drop_at_drop_pos_c = check_entry2(_("Insert drag and drop at drop position\ninstead of cursor position (experimental)"), main_v->props.drop_at_drop_pos, vbox1);
 
@@ -397,14 +414,14 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	conv_shift_enter_c = check_entry2(_("Insert text with Shift-Enter"), main_v->props.conv_shift_enter, vbox2);
 	tmpstr = escapestring(main_v->props.shift_enter_text, 0);
 	if (tmpstr)
-		shift_enter_text_c = string_entry2("  ", tmpstr, vbox2, none);
+		shift_enter_text_c = string_entry2("  ", tmpstr, vbox2, none, dialog);
 	g_free(tmpstr);
 	conv_ctrl_enter_c = check_entry2(_("Insert text with Ctrl-Enter"), main_v->props.conv_ctrl_enter, vbox2);
 	tmpstr = escapestring(main_v->props.ctrl_enter_text, 0);
 	if (tmpstr)
-		ctrl_enter_text_c = string_entry2("  ", tmpstr, vbox2, none);
+		ctrl_enter_text_c = string_entry2("  ", tmpstr, vbox2, none, dialog);
 	g_free(tmpstr);
-	conv_special_char_c = check_entry2(_("Insert HTML entity with Ctrl-(special char)\n(&lt; &gt; &amp;)"), main_v->props.conv_special_char, vbox2);
+	conv_special_char_c = check_entry2(_("Insert HTML entity with Ctrl-(special char)\n(&lt; &gt; &amp; &quot;)"), main_v->props.conv_special_char, vbox2);
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, gtk_label_new(_("Editor")));
 
@@ -422,8 +439,8 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	poplist = g_list_append(poplist, "XHTML 1.0 Transitional");
 	poplist = g_list_append(poplist, "XHTML Basic");
 	poplist = g_list_append(poplist, "XHTML 1.1");
-	poplist = g_list_append(poplist, "allow everything");
-	poplist = g_list_append(poplist, "Read from File");
+	poplist = g_list_append(poplist, N_("allow everything"));
+	poplist = g_list_append(poplist, N_("Read from File"));
 	html_ver_c = combo_entry2(_("used HTML-Version"), main_v->props.html_ver, vbox, poplist);
 	g_list_free(poplist);
 
@@ -449,14 +466,14 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
-	tabfont_e = string_entry2(_("Notebook tab fontset"), main_v->props.cfg_tab_font, vbox, font);
+	tabfont_e = string_entry2(_("Notebook tab fontset"), main_v->props.cfg_tab_font, vbox, font, dialog);
 	fontset_c = check_entry2(_("Font is a fontset"), main_v->props.fontset, vbox);
 
 	poplist = NULL;
-	poplist = g_list_append(poplist, "top");
-	poplist = g_list_append(poplist, "bottom");
-	poplist = g_list_append(poplist, "left");
-	poplist = g_list_append(poplist, "right");
+	poplist = g_list_append(poplist, N_("top"));
+	poplist = g_list_append(poplist, N_("bottom"));
+	poplist = g_list_append(poplist, N_("left"));
+	poplist = g_list_append(poplist, N_("right"));
 
 	tabpos_c = combo_entry2(_("Notebook position"), main_v->props.cfg_tab_pos, vbox, poplist);
 	g_list_free(poplist);
@@ -478,6 +495,10 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	g_list_free(poplist);
 
 	transient_htdialogs_c = check_entry2(_("Keep HTML dialogs on top"), main_v->props.transient_htdialogs, vbox);
+#ifdef AUTOCOMPLET
+	autocomplet_html_c = check_entry2(_("Show autocompletion window for HTML tags while editing"), main_v->props.autocomplet_html, vbox);
+	autocomplet_delay_s = spin_button2(_("Delay to show the autocompletion window"), main_v->props.autocomplet_delay, vbox, 0.0, 10000.0);
+#endif
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, gtk_label_new(_("GUI")));
 
@@ -495,11 +516,20 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	thumbtype_c = combo_entry2(_("Thumbnail type"), main_v->props.cfg_thumbnailtype, vbox, poplist);
 	g_list_free(poplist);
 
-	thumbstring_e = string_entry2(_("Thumbnail extension string"), main_v->props.cfg_thumbnailstring, vbox, none);
+	thumbstring_e = string_entry2(_("Thumbnail extension string"), main_v->props.cfg_thumbnailstring, vbox, none, dialog);
 
 	allow_multi_instances_c = check_entry2(_("Allow multiple instances of the same file"), main_v->props.allow_multi_instances, vbox);
 	backup_file_c = check_entry2(_("Backup file before saving"), main_v->props.backup_file, vbox);
-	backup_filestring_e = string_entry2(_("Backup file suffix"), main_v->props.backup_filestring, vbox, none);
+	backup_filestring_e = string_entry2(_("Backup file suffix"), main_v->props.backup_filestring, vbox, none, dialog);
+	backup_by_copy_c = check_entry2(_("Backup file by copy instead of rename"), main_v->props.backup_by_copy, vbox);
+
+	poplist = NULL;
+	poplist = g_list_append(poplist, N_("save"));
+	poplist = g_list_append(poplist, N_("abort"));
+	poplist = g_list_append(poplist, N_("ask"));
+	backup_abort_style_c = combo_entry2(_("Backup failure action"), main_v->props.backup_abort_style, vbox, poplist);
+	g_list_free(poplist);
+
 #ifdef WITH_MSG_QUEUE
 	open_in_running_bluefish_c = check_entry2(_("Open commandline files in already running bluefish"), main_v->props.open_in_running_bluefish, vbox);
 #endif
@@ -508,12 +538,12 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
-	filebrowser_filter_image_e = string_entry2(_("Image filter"), main_v->props.filebrowser_filter_image, vbox, none);
-	filebrowser_filter_php_e = string_entry2(_("PHP filter"), main_v->props.filebrowser_filter_php, vbox, none);
-	filebrowser_filter_html_e = string_entry2(_("HTML filter"), main_v->props.filebrowser_filter_html, vbox, none);
-	filebrowser_filter_jscript_e = string_entry2(_("JavaScript filter"), main_v->props.filebrowser_filter_jscript, vbox, none);
-	filebrowser_filter_css_e = string_entry2(_("Cascading stylesheet filter"), main_v->props.filebrowser_filter_css, vbox, none);
-	filebrowser_filter_other_e = string_entry2(_("Other webfiles filter"), main_v->props.filebrowser_filter_other, vbox, none);
+	filebrowser_filter_image_e = string_entry2(_("Image filter"), main_v->props.filebrowser_filter_image, vbox, none, dialog);
+	filebrowser_filter_php_e = string_entry2(_("PHP filter"), main_v->props.filebrowser_filter_php, vbox, none, dialog);
+	filebrowser_filter_html_e = string_entry2(_("HTML filter"), main_v->props.filebrowser_filter_html, vbox, none, dialog);
+	filebrowser_filter_jscript_e = string_entry2(_("JavaScript filter"), main_v->props.filebrowser_filter_jscript, vbox, none, dialog);
+	filebrowser_filter_css_e = string_entry2(_("Cascading stylesheet filter"), main_v->props.filebrowser_filter_css, vbox, none, dialog);
+	filebrowser_filter_other_e = string_entry2(_("Other webfiles filter"), main_v->props.filebrowser_filter_other, vbox, none, dialog);
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmpvbox, gtk_label_new(_("Files and images")));
 
@@ -525,8 +555,9 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
-	browser_e = string_entry2(_("Browser command"), main_v->props.cfg_browser_cline, vbox, file);
-	weblint_e = string_entry2(_("Weblint command"), main_v->props.cfg_weblint_cline, vbox, file);
+	browser_e = string_entry2(_("Browser command"), main_v->props.cfg_browser_cline, vbox, file, dialog);
+	weblint_e = string_entry2(_("Weblint command"), main_v->props.cfg_weblint_cline, vbox, file, dialog);
+	image_editor_cline_e = string_entry2(_("Image editor command"), main_v->props.image_editor_cline, vbox, file, dialog);
 
 #ifdef WITH_SPC
 	frame = gtk_frame_new(_("Spell checker options"));
@@ -534,14 +565,14 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
-	spc_command_e = string_entry2(_("Ispell command"), main_v->props.cfg_spc_cline, vbox,1 );
-	spc_lang_e = string_entry2(_("Default language"), main_v->props.cfg_spc_lang, vbox, 0);
+	spc_command_e = string_entry2(_("Ispell command"), main_v->props.cfg_spc_cline, vbox,file, dialog );
+	spc_lang_e = string_entry2(_("Default language"), main_v->props.cfg_spc_lang, vbox, none, dialog);
 	spc_acc_comp_c = check_entry2(_("Accept compound words"), main_v->props.spc_accept_compound, vbox);
 	spc_use_esc_c = check_entry2(_("Specify additional characters that might form a word"), main_v->props.spc_use_esc_chars, vbox);
 	spc_use_pers_dict_c = check_entry2(_("Use personal dictionary"), main_v->props.spc_use_pers_dict, vbox);
-	spc_pers_dict_e = string_entry2(_("Personal dictionary"), main_v->props.spc_pers_dict, vbox, 1);
+	spc_pers_dict_e = string_entry2(_("Personal dictionary"), main_v->props.spc_pers_dict, vbox, file, dialog);
    spc_use_input_encoding_c = check_entry2(_("Use input encoding"), main_v->props.spc_use_input_encoding, vbox);
-	spc_input_encoding_e = string_entry2(_("Input Encoding"), main_v->props.spc_input_encoding, vbox, 0);
+	spc_input_encoding_e = string_entry2(_("Input Encoding"), main_v->props.spc_input_encoding, vbox, none, dialog);
    spc_output_html_chars_c = check_entry2(_("Output html characters"), main_v->props.spc_output_html_chars, vbox);
 #endif
 
@@ -560,11 +591,10 @@ void configure_cb(GtkWidget * widget, gpointer page)
 	gtk_box_pack_end(GTK_BOX(ahbox), cancel, TRUE, TRUE, 0);
 
 	/* show the widgets */
-	DEBUG_MSG("Opening page %i \n",GPOINTER_TO_INT(page));
-        gtk_notebook_set_page (GTK_NOTEBOOK(notebook),GPOINTER_TO_INT(page));
+
 	gtk_widget_show_all(dialog);
+	gtk_notebook_set_page (GTK_NOTEBOOK(notebook),GPOINTER_TO_INT(page));
 	gtk_grab_add(dialog);
-	
 	gtk_main();
 	DEBUG_MSG("configure_cb, just after gtk_main()\n");
   	statusbar_remove(GINT_TO_POINTER(count));
@@ -575,6 +605,7 @@ static void integer_apply(gint * config_var, GtkWidget * spinbutton)
 {
 
 	*config_var = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));
+	DEBUG_MSG("integer_apply, config_var(%p)=%i\n", config_var, *config_var);
 }
 
 static void check_apply(gint * config_var, GtkWidget * check_button)
@@ -585,5 +616,6 @@ static void check_apply(gint * config_var, GtkWidget * check_button)
 	} else {
 		*config_var = 0;
 	}
+	DEBUG_MSG("check_apply, config_var(%p)=%i\n", config_var, *config_var);
 }
 
