@@ -28,6 +28,9 @@
 #include <glib.h>
 #include <gtk/gtkwidget.h>
 
-void configure_cb(GtkWidget * widget, gpointer data);
+void configure_cb(GtkWidget * widget, gpointer page);
+/* page: At which page do you want to open the configuration notebook ? */
+
+#define CONFIGURE_EXTERNAL_PAGE  GINT_TO_POINTER(4)
 
 #endif							/* __CONFIGURE_H_ */
