@@ -26,8 +26,8 @@
 /* If GNU C compiler, then we can fully remove all debug info */
 #ifdef __GNUC__
 #define DEBUG_MSG(format, args...)
-/**/
-#else /* !__GNUC__ */
+ /**/
+#else							/* !__GNUC__ */
 #define DEBUG_MSG g_none
 /*
  * Function: g_none
@@ -39,15 +39,14 @@
  * Description:
  *      none
  */
-static void 
-g_none (gchar * format,...)
+static void g_none(gchar * format, ...)
 {
-  return;
+	return;
 };
+
 #endif
 
 #endif
 
 
-#endif	/* __DEBUG_H_ */
-
+#endif							/* __DEBUG_H_ */
