@@ -20,9 +20,15 @@
 #ifndef DEFAULT_INCLUDE_H
 #define DEFAULT_INCLUDE_H
 
+#include "config.h"
+
 #include <gtk/gtk.h>
 
-#include "config.h"
+#ifdef ENABLE_NLS
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+#endif
 
 #define NEW_CS3
 
