@@ -1,6 +1,6 @@
 /* Bluefish HTML Editor
  *
- * Copyright (C) 1999-2000 Olivier Sessink
+ * Copyright (C) 1999-2001 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,15 +18,22 @@
  */
 #include "default_include.h"
 
-#include <locale.h>
-
 #include <stdlib.h> /* strtod() */
+#include <string.h> /* strlen() */
 
 #include "gtk_easy.h"
 #include "bluefish.h"
 #include "bf_lib.h"
 #include "pixmaps.h"
 
+#ifdef WIN32
+#define DIRSTR "\\"
+#define DIRCHR 92
+#else
+#define DIRSTR "/"
+#define DIRCHR '/'
+#endif
+
 /* void flush_queue(void)
  * handles the gtk and gdk queue
  */
@@ -119,13 +126,18 @@ void string_apply(gchar ** config_var, GtkWidget * entry)
 	DEBUG_MSG("string_apply, start\n");
 
 	tmpstring = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
-	DEBUG_MSG("string_apply, tmpstring=%s\n", tmpstring);
+	DEBUG_MSG("string_apply, tmpstring(%p)=%s\n", tmpstring, tmpstring);
 	if (tmpstring) {
 		if (config_var != NULL) {
 			g_free(*config_var);
 		}
 		*config_var = tmpstring;
 	}
+#ifdef DEBUG
+	else {
+		DEBUG_MSG("string_apply, ***** tmpstring == NULL *****\n", tmpstring, tmpstring);
+	}
+#endif
 }
 
 /*
@@ -213,6 +225,17 @@ GtkWidget *boxed_entry_with_text(gchar * setstring, gint max_lenght, GtkWidget *
 
 }
 
+GtkWidget *boxed_full_entry(gchar * labeltext, gchar * setstring,gint max_lenght, GtkWidget * box) {
+	GtkWidget *hbox, *return_widget;
+
+	hbox = gtk_hbox_new(FALSE,3);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(labeltext), FALSE, FALSE, 3);
+	return_widget = boxed_entry_with_text(setstring, max_lenght, hbox);
+	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 3);
+
+	return return_widget;
+}
+
 GtkWidget *checkbut_with_value(gchar *labeltext, gint which_config_int) {
 	GtkWidget *returnwidget;
 
@@ -686,7 +709,7 @@ static void fs_ok_clicked_lcb(GtkWidget * widget, Tfileselect *fileselect)
 			if (GTK_CLIST_ROW(row)->state == GTK_STATE_SELECTED) {
 				gtk_clist_get_text(GTK_CLIST(GTK_FILE_SELECTION(fileselect->fs)->file_list), rownum, 0, &temp);
 				DEBUG_MSG("fs_ok_clicked_lcb, temp=%s\n", temp);
-				totalfilename = g_strconcat(dirname, "/", temp, NULL);
+				totalfilename = g_strconcat(dirname, DIRSTR, temp, NULL);
 				fileselect->filenames_to_return = g_list_append(fileselect->filenames_to_return, totalfilename);
 			}
 			DEBUG_MSG("fs_ok_clicked_lcb, rownum=%d\n", rownum);
@@ -699,6 +722,14 @@ static void fs_ok_clicked_lcb(GtkWidget * widget, Tfileselect *fileselect)
 		/* NO multiple files allowed --> return just one file */
 		fileselect->filename_to_return = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect->fs)));
 	}
+	if ((fileselect->multipleselect 
+			&& (g_list_length(fileselect->filenames_to_return) < 1)) 
+		|| (!fileselect->multipleselect 
+			&& strlen(fileselect->filename_to_return) < 1)){
+		error_dialog(_("Bluefish error"), _("No filename known"));
+	}
+	
+	
 	gtk_main_quit();
 	window_destroy(GTK_WIDGET(fileselect->fs));
 }
@@ -755,6 +786,4 @@ GList *return_files(gchar * setfile) {
 	return return_files_w_title(setfile, _("Select files"));
 }
 
-
-
 /************************************************************************/
