@@ -31,6 +31,8 @@ GTK_WINDOW_TOPLEVEL
 GTK_WINDOW_DIALOG
 GTK_WINDOW_POPUP */
 
+typedef enum { none, file, font } Textra_but;
+
 void error_dialog(gchar * window_title, gchar * error_string);
 void setup_toggle_item(GtkItemFactory * ifactory, gchar * path, gint state);
 void string_apply(gchar ** config_var, GtkWidget * entry);
@@ -39,6 +41,7 @@ GtkWidget *boxed_combo_with_popdown(gchar * setstring, GList * which_list, gint
 GtkWidget *combo_with_popdown_sized(gchar * setstring, GList * which_list, gint editable, gint width);
 GtkWidget *entry_with_text(gchar * setstring, gint max_lenght);
 GtkWidget *boxed_entry_with_text(gchar * setstring, gint max_lenght, GtkWidget *box);
+GtkWidget *boxed_full_entry(gchar * labeltext, gchar * setstring,gint max_lenght, GtkWidget * box);
 GtkWidget *checkbut_with_value(gchar *labeltext, gint which_config_int);
 GtkWidget *boxed_checkbut_with_value(gchar *labeltext, gint which_config_int, GtkWidget * box);
 GtkWidget *radiobut_with_value(gchar *labeltext, gint enabled, GtkRadioButton *prevbut);
@@ -61,5 +64,4 @@ gchar *return_file_w_title(gchar * setfile, gchar *title);
 gchar *return_file(gchar * setfile);
 GList *return_files_w_title(gchar * setfile, gchar *title);
 GList *return_files(gchar * setfile);
-
 #endif
