@@ -22,7 +22,7 @@
  */
 /* 
  * Changes by Antti-Juhani Kaijanaho <gaia@iki.fi> on 1999-10-20
- * $Id: html.c,v 1.52 2000/12/13 22:04:33 olivier Exp $
+ * $Id: html.c,v 1.2 2001/09/09 21:47:21 oli4 Exp $
  */
 #include "default_include.h"
  
@@ -32,8 +32,6 @@
 #include <stdlib.h> /* strtod() */
 #include <time.h>
 
-#include <locale.h>
-
 #include "html.h" 	/* myself */
 #include "html_diag.h" 	/* the new html dialog stuff  */
 #include "html2.h" /* style_but_new*/
@@ -45,7 +43,6 @@
 #include "gtk_easy.h"
 #include "coloursel.h" /* color_but_new () */
 #include "toolbars.h"
-#include "undo.h"
 #include "document.h"
 #include "images.h"
 #include "callbacks.h"
@@ -203,12 +200,12 @@ void general_html_cb(GtkWidget * widget, gpointer data)
 		insert_dbl_text(cap("<NOBR>"), cap("</NOBR>"));
 		break;
 	case 44:
-		tmp = g_strdup_printf("<%s=\"javascript\"><!--\n", cap("SCRIPT LANGUAGE"));
+		tmp = g_strdup_printf("<%s=\"javascript\" %s=\"text/javascript\"><!--\n", cap("SCRIPT LANGUAGE"), cap("TYPE"));
 		insert_dbl_text(tmp, cap("\n//--></SCRIPT>"));
 		g_free(tmp);
 		break;
 /*	case 45:
-		tmp = g_strdup_printf("<%s=\"javascript\" %s=\"\">", cap("SCRIPT LANGUAGE"), cap("SRC"));
+		tmp = g_strdup_printf("<%s=\"javascript\" %s>", cap("SCRIPT LANGUAGE"), cap("SRC=\"\" TYPE=\"text/javascript\""));
 		insert_dbl_text(tmp, cap("</SCRIPT>"));
 		g_free(tmp);
 		break; */
@@ -218,7 +215,9 @@ void general_html_cb(GtkWidget * widget, gpointer data)
 		g_free(tmp);
 		break; */
 	case 47:
-		tmp = g_strconcat(cap("<META NAME=\"Generator\" CONTENT=\""), "Bluefish ",  VERSION, "\">\n", NULL);
+		tmp = g_strconcat(cap("<META NAME=\"Generator\" CONTENT=\"")
+			, "Bluefish ",  VERSION
+			, main_v->props.xhtml == 1 ? "\" />\n" : "\">\n" , NULL);
 		insert_dbl_text(tmp, NULL);
 		g_free(tmp);
 		break;
@@ -336,7 +335,7 @@ void insert_time_cb(GtkWidget * widget, gpointer data)
 	for (count = 1; count < 6; count++) {
 		switch (count) {
 		case 1:
-			temp = g_strdup_printf("  Time (%i:%i:%i)", time_struct->tm_hour, time_struct->tm_min, time_struct->tm_sec);
+			temp = g_strdup_printf(_("  Time (%i:%i:%i)"), time_struct->tm_hour, time_struct->tm_min, time_struct->tm_sec);
 			break;
 		case 2:
 			switch (time_struct->tm_wday) {
@@ -499,7 +498,7 @@ static void quickanchorok_lcb(GtkWidget * widget, Thtml_diag * dg)
 	main_v->current_project.classlist = add_entry_to_stringlist(main_v->current_project.classlist, GTK_WIDGET(GTK_COMBO(dg->combo[3])->entry));
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</A>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</A>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -527,7 +526,16 @@ void quickanchor_cb(GtkWidget * widget, gpointer data)
 	dgtable = gtk_table_new(4, 10, 0);
 	gtk_box_pack_start(GTK_BOX(lvbox), dgtable, FALSE, FALSE, 0);
 
-	dg->combo[2] = combo_with_popdown(avalues[0], main_v->current_project.urllist, 1);
+	{
+	GList *rel_link_list, *tmplist;
+	rel_link_list = generate_relative_doc_list();
+	tmplist = duplicate_stringlist(main_v->current_project.urllist, 1);
+	rel_link_list = g_list_concat(tmplist, rel_link_list);
+
+	dg->combo[2] = combo_with_popdown(avalues[0], rel_link_list, 1);
+
+	free_stringlist(rel_link_list);
+	}
 	file_but = file_but_new(GTK_WIDGET(GTK_COMBO(dg->combo[2])->entry), dg->dialog, 0);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(file_but), 9, 10, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("HREF")), 0, 1, 0, 1);
@@ -669,7 +677,7 @@ static void block_tag_editok_lcb(gint type, Thtml_diag * dg) {
 	main_v->current_project.classlist = add_entry_to_stringlist(main_v->current_project.classlist, GTK_WIDGET(GTK_COMBO(dg->combo[2])->entry));
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, endstring);
+		doc_insert_dbl_text(dg->doc, finalstring, endstring);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -754,8 +762,7 @@ void block_tag_edit_cb(gint type, GtkWidget *widget, gpointer data)
 	g_free(labeltext);
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 12, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 12);
 	
 	popuplist = g_list_append(NULL, "left");
 	popuplist = g_list_append(popuplist, "center");
@@ -844,7 +851,7 @@ static void quickruleok_lcb(GtkWidget * widget, Thtml_diag * dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -867,8 +874,7 @@ void quickrule_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Horizontal Rule"));
 	fill_dialogvalues(hritems, hrvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	popdownlist = g_list_insert(popdownlist, "center", 0);
 	popdownlist = g_list_insert(popdownlist, "left", 1);
@@ -921,7 +927,7 @@ static void quickstart_ok_lcb(GtkWidget * widget, Thtml_diag * dg) {
 		add_to_stringlist(main_v->current_project.dtd_cblist, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dg->combo[1])->entry)));
 
 	if(main_v->props.xhtml){
-		tmpchar1 = g_strdup_printf("%s\n%shttp://www.w3.org/1999/xhtml%sde%sde\">\n%s\n", gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dg->combo[1])->entry)), cap("<HTML XMLNS=\""), cap("\" XML:LANG=\""), cap("\" LANG=\""), cap("<HEAD>"));
+		tmpchar1 = g_strdup_printf("%s\n%shttp://www.w3.org/1999/xhtml%sen%sen\">\n%s\n", gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dg->combo[1])->entry)), cap("<HTML XMLNS=\""), cap("\" XML:LANG=\""), cap("\" LANG=\""), cap("<HEAD>"));
 	} else{
 		tmpchar1 = g_strdup_printf("%s\n%s\n", gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dg->combo[1])->entry)), cap("<HTML>\n<HEAD>"));
 	}
@@ -942,7 +948,7 @@ static void quickstart_ok_lcb(GtkWidget * widget, Thtml_diag * dg) {
 	finalstring = g_strconcat(tmpchar1, cap("</HEAD>\n<BODY>"), NULL);
 	g_free(tmpchar1);
 
-	insert_dbl_text(finalstring, cap("</BODY>\n</HTML>"));
+	doc_insert_dbl_text(dg->doc, finalstring, cap("</BODY>\n</HTML>"));
 
 	g_free(finalstring);
 	html_diag_destroy_cb(NULL, NULL, dg);
@@ -957,9 +963,7 @@ void quickstart_cb(GtkWidget * widget, gpointer data)
 	
 	dg = html_diag_new(_("Quick Start"));
 
-	dgtable = gtk_table_new(7, 4, 0);
-	gtk_table_set_col_spacings(GTK_TABLE(dgtable), 10);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 7, 4);
 
 	main_v->current_project.dtd_cblist = add_to_stringlist(main_v->current_project.dtd_cblist, "");
 	main_v->current_project.dtd_cblist = add_to_stringlist(main_v->current_project.dtd_cblist, "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 3.2//EN\">");
@@ -982,7 +986,7 @@ void quickstart_cb(GtkWidget * widget, gpointer data)
 	
 	scrolwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new("Header tags"), 0, 1, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Header tags")), 0, 1, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), scrolwin, 1, 4, 1, 4);
 
 	dg->clist[1] = gtk_clist_new(1);
@@ -1010,24 +1014,34 @@ static void bodyok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	gchar *thestring, *finalstring;
 
 	thestring = g_strdup(cap("<BODY"));
-	thestring = insert_string_if_entry(dg->entry[1], cap("BACKGROUND"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[1])->entry), cap("BGCOLOR"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[2])->entry), cap("TEXT"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[3])->entry), cap("LINK"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[4])->entry), cap("VLINK"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[5])->entry), cap("ALINK"), thestring, NULL);
+	if (dg->entry[1]) {
+		thestring = insert_string_if_entry(dg->entry[1], cap("BACKGROUND"), thestring, NULL);
+		thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[1])->entry), cap("BGCOLOR"), thestring, NULL);
+		thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[2])->entry), cap("TEXT"), thestring, NULL);
+		thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[3])->entry), cap("LINK"), thestring, NULL);
+		thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[4])->entry), cap("VLINK"), thestring, NULL);
+		thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[5])->entry), cap("ALINK"), thestring, NULL);
+	}
+	thestring = insert_string_if_entry(dg->entry[3], cap("STYLE"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[6])->entry), cap("CLASS"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[4], cap("ID"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[5], cap("LANG"), thestring, NULL);
+
 	thestring = insert_string_if_entry(dg->entry[2], NULL, thestring, NULL);
 	finalstring = g_strconcat(thestring, ">\n", NULL);
 	g_free(thestring);
 
-	main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.classlist, GTK_WIDGET(GTK_COMBO(dg->combo[1])->entry));
-	main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.classlist, GTK_WIDGET(GTK_COMBO(dg->combo[2])->entry));
-	main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.classlist, GTK_WIDGET(GTK_COMBO(dg->combo[3])->entry));
-	main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.classlist, GTK_WIDGET(GTK_COMBO(dg->combo[4])->entry));
-	main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.classlist, GTK_WIDGET(GTK_COMBO(dg->combo[5])->entry));
+	if (dg->entry[1]) {
+		main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.colorlist, GTK_WIDGET(GTK_COMBO(dg->combo[1])->entry));
+		main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.colorlist, GTK_WIDGET(GTK_COMBO(dg->combo[2])->entry));
+		main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.colorlist, GTK_WIDGET(GTK_COMBO(dg->combo[3])->entry));
+		main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.colorlist, GTK_WIDGET(GTK_COMBO(dg->combo[4])->entry));
+		main_v->current_project.colorlist = add_entry_to_stringlist(main_v->current_project.colorlist, GTK_WIDGET(GTK_COMBO(dg->combo[5])->entry));
+	}
+	main_v->current_project.classlist = add_entry_to_stringlist(main_v->current_project.classlist, GTK_WIDGET(GTK_COMBO(dg->combo[6])->entry));
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</BODY>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -1038,69 +1052,112 @@ static void bodyok_lcb(GtkWidget * widget, Thtml_diag *dg)
 
 void body_cb(GtkWidget * widget, gpointer data)
 {
-	static gchar *bodyitems[] = { "background", "bgcolor", "text", "link", "vlink", "alink", NULL };
+	static gchar *bodyitems[] = { "background", "bgcolor", "text", "link", "vlink", "alink"
+									, "style", "class", "id", "language", "onload", "onunload", NULL };
 
-	GtkWidget *color_but, *file_but, *dgtable;
-	gchar *bodyvalues[7];
+	GtkWidget *color_but, *file_but, *dgtable[3], *stylebut, *noteb, *hbox;
+	gchar *bodyvalues[11];
 	gchar *custom = NULL;
 	Thtml_diag *dg;
 
-	if(!main_v->props.allow_dep && widget && !data){
-		insert_dbl_text(cap("<BODY>\n"), cap("\n</BODY>"));
-	} else {
+	dg = html_diag_new(_("Body"));
+	fill_dialogvalues(bodyitems, bodyvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-		dg = html_diag_new(_("Body"));
-		fill_dialogvalues(bodyitems, bodyvalues, &custom, (Ttagpopup *) data, widget, dg);
+	noteb = gtk_notebook_new();
+	gtk_box_pack_start(GTK_BOX(dg->vbox), noteb, FALSE, FALSE, 0);
 
-		dgtable = gtk_table_new(5, 10, 0);
-		gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+		dgtable[1] = gtk_table_new(5, 10, 0);
+		gtk_notebook_append_page(GTK_NOTEBOOK(noteb), dgtable[1], gtk_label_new(_("Options")));
 
+		dg->entry[3] = entry_with_text(bodyvalues[6], 256);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), dg->entry[3], 1, 9, 0, 1);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), gtk_label_new(_("Style")), 0, 1, 0, 1);
+#ifdef NEW_CS3
+		stylebut = style_but_new(dg->entry[3], dg->dialog);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), stylebut, 9, 10, 0, 1);
+#endif /* #ifdef NEW_CS3 */
+		dg->combo[6] = combo_with_popdown(bodyvalues[7], main_v->current_project.classlist, 1);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), dg->combo[6], 1, 10, 1, 2);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), gtk_label_new(_("Class")), 0, 1, 1, 2);
+
+		dg->entry[4] = entry_with_text(bodyvalues[8], 256);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), dg->entry[4], 1, 10, 2, 3);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), gtk_label_new(_("Id")), 0, 1, 2, 3);
+
+		dg->entry[5] = entry_with_text(bodyvalues[9], 256);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), dg->entry[5], 1, 10, 3, 4);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[1]), gtk_label_new(_("Language")), 0, 1, 3, 4);
+
+		dgtable[2] = gtk_table_new(5, 10, 0);
+		gtk_notebook_append_page(GTK_NOTEBOOK(noteb), dgtable[2], gtk_label_new(_("Events")));
+		
+		dg->entry[6] = entry_with_text(bodyvalues[10], 256);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[2]), gtk_label_new(_("OnLoad")), 0, 1, 0, 1);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[2]), dg->entry[6], 1, 10, 0, 1);
+
+		dg->entry[7] = entry_with_text(bodyvalues[11], 256);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[2]), gtk_label_new(_("OnUnLoad")), 0, 1, 1, 2);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[2]), dg->entry[7], 1, 10, 1, 2);
+
+	if (!(!main_v->props.allow_dep && widget && !data)){
+		dgtable[0] = gtk_table_new(5, 10, 0);
+		gtk_notebook_append_page(GTK_NOTEBOOK(noteb), dgtable[0], gtk_label_new(_("Deprecated options")));
 
 		dg->entry[1] = entry_with_text(bodyvalues[0], 256);
 		file_but = file_but_new(dg->entry[1], dg->dialog, 0);
 
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(file_but), 9, 10, 0, 1);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Background Image")), 0, 1, 0, 1);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 9, 0, 1);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), GTK_WIDGET(file_but), 9, 10, 0, 1);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), gtk_label_new(_("Background Image")), 0, 1, 0, 1);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->entry[1], 1, 9, 0, 1);
 
 		dg->combo[1] = combo_with_popdown(bodyvalues[1], main_v->current_project.colorlist, 1);
 		color_but = color_but_new(GTK_COMBO(dg->combo[1])->entry, dg->dialog);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(color_but), 9, 10, 1, 2);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[1])), 1, 9, 1, 2);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Background Color")), 0, 1, 1, 2);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), color_but, 9, 10, 1, 2);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[1], 1, 9, 1, 2);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), gtk_label_new(_("Background Color")), 0, 1, 1, 2);
 
 		dg->combo[2] = combo_with_popdown(bodyvalues[2], main_v->current_project.colorlist, 1);
 		color_but = color_but_new(GTK_COMBO(dg->combo[2])->entry, dg->dialog);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(color_but), 9, 10, 2, 3);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[2])), 1, 9, 2, 3);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Text Color")), 0, 1, 2, 3);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), color_but, 9, 10, 2, 3);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[2], 1, 9, 2, 3);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), gtk_label_new(_("Text Color")), 0, 1, 2, 3);
 
 		dg->combo[3] = combo_with_popdown(bodyvalues[3], main_v->current_project.colorlist, 1);
 		color_but = color_but_new(GTK_COMBO(dg->combo[3])->entry, dg->dialog);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(color_but), 9, 10, 3, 4);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[3])), 1, 9, 3, 4);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Link Color")), 0, 1, 3, 4);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), color_but, 9, 10, 3, 4);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[3], 1, 9, 3, 4);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), gtk_label_new(_("Link Color")), 0, 1, 3, 4);
 
 		dg->combo[4] = combo_with_popdown(bodyvalues[4], main_v->current_project.colorlist, 1);
 		color_but = color_but_new(GTK_COMBO(dg->combo[4])->entry, dg->dialog);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(color_but), 9, 10, 4, 5);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[4])), 1, 9, 4, 5);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Visited Link Color")), 0, 1, 4, 5);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), color_but, 9, 10, 4, 5);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[4], 1, 9, 4, 5);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), gtk_label_new(_("Visited Link Color")), 0, 1, 4, 5);
 
 		dg->combo[5] = combo_with_popdown(bodyvalues[5], main_v->current_project.colorlist, 1);
 		color_but = color_but_new(GTK_COMBO(dg->combo[5])->entry, dg->dialog);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(color_but), 9, 10, 5, 6);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[5])), 1, 9, 5, 6);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Active Link Color")), 0, 1, 5, 6);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), color_but, 9, 10, 5, 6);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), dg->combo[5], 1, 9, 5, 6);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable[0]), gtk_label_new(_("Active Link Color")), 0, 1, 5, 6);
 
-		dg->entry[2] = entry_with_text(custom, 1024);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(dg->entry[2]), 1, 9, 6, 7);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Custom values")), 0, 1, 6, 7);
-
-		html_diag_finish(dg, bodyok_lcb);
-
-		if (custom)	g_free(custom);
+	} else {
+		dg->combo[1] = NULL;
+		dg->combo[2] = NULL;
+		dg->combo[3] = NULL;
+		dg->combo[4] = NULL;
+		dg->combo[5] = NULL;
+		dg->entry[1] = NULL;
 	}
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(dg->vbox), hbox, FALSE, FALSE, 0);
+	dg->entry[2] = entry_with_text(custom, 1024);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Custom values")), FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), dg->entry[2], TRUE, TRUE, 0);
+
+	html_diag_finish(dg, bodyok_lcb);
+
+	if (custom)	g_free(custom);
 }
 
 static void metaok_lcb(GtkWidget * widget, Thtml_diag *dg)
@@ -1122,7 +1179,7 @@ static void metaok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -1144,9 +1201,7 @@ void meta_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Meta"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_table_set_col_spacings(GTK_TABLE(dgtable), 10);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	popuplist = g_list_append(NULL, "Description");
 	popuplist = g_list_append(popuplist, "Date");
@@ -1231,7 +1286,7 @@ static void generalfontdialog_lcb(gint type, GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</FONT>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</FONT>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -1270,8 +1325,7 @@ static void generalfontdialog_cb(gint type, GtkWidget * widget, gpointer data)
 	}
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);		
 
-	dgtable = gtk_table_new(3, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 3, 10);
 
 	popdownlist = g_list_insert(popdownlist, "+", 0);
 	popdownlist = g_list_insert(popdownlist, "-", 1);
@@ -1319,11 +1373,25 @@ void basefont_cb(GtkWidget *widget, gpointer data) {
 
 static void emailok_lcb(GtkWidget * widget, Thtml_diag *dg)
 {
-	gchar *finalstring;
-	finalstring = g_strconcat(cap("<A HREF=\"mailto:")
+	gchar *finalstring, *tmpstr, *subj, *body;
+	tmpstr = g_strconcat(cap("<A HREF=\"mailto:")
 			, gtk_entry_get_text(GTK_ENTRY(dg->entry[1]))
-			, "\">", NULL);
-	insert_dbl_text(finalstring, cap("</A>"));
+			, NULL);
+	if (strlen(gtk_entry_get_text(GTK_ENTRY(dg->entry[2])))) {
+		subj = g_strconcat("?Subject=", gtk_entry_get_text(GTK_ENTRY(dg->entry[2])), NULL);
+	} else {
+		subj = g_strdup("");
+	}
+	if (strlen(gtk_entry_get_text(GTK_ENTRY(dg->entry[3])))) {
+		body = g_strconcat("?Body=", gtk_entry_get_text(GTK_ENTRY(dg->entry[3])), NULL);
+	} else {
+		body = g_strdup("");
+	}
+	finalstring = g_strconcat(tmpstr, subj, body, "\">", NULL);
+	g_free(tmpstr);
+	g_free(subj);
+	g_free(body);
+	doc_insert_dbl_text(dg->doc, finalstring, cap("</A>"));
 	g_free(finalstring);
 	html_diag_destroy_cb(NULL, NULL, dg);
 }
@@ -1335,12 +1403,21 @@ void email_cb(GtkWidget * widget, gpointer data)
 	
 	dg = html_diag_new(_("Email"));
 	
-	hbox = gtk_hbox_new(FALSE, 5);
+	hbox = gtk_vbox_new(FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(dg->vbox), hbox, FALSE, FALSE, 0);
 
 	dg->entry[1] = gtk_entry_new_with_max_length(256);
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Email address:")), FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), dg->entry[1], FALSE, FALSE, 0);
+
+	dg->entry[2] = gtk_entry_new_with_max_length(256);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("UrlEncoded subject:")), FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), dg->entry[2], FALSE, FALSE, 0);
+
+	dg->entry[3] = gtk_entry_new_with_max_length(256);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("UrlEncoded body:")), FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), dg->entry[3], FALSE, FALSE, 0);
+
 	html_diag_finish(dg, emailok_lcb);
 }
 
@@ -1369,9 +1446,9 @@ static void quicklistok_lcb(GtkWidget * widget, Thtml_diag *dg)
 		strcat(finalstring, "\n");
 
 		if (active) {
-			insert_dbl_text(finalstring, cap("</OL>"));
+			doc_insert_dbl_text(dg->doc, finalstring, cap("</OL>"));
 		} else {
-			insert_dbl_text(finalstring, cap("</UL>"));
+			doc_insert_dbl_text(dg->doc, finalstring, cap("</UL>"));
 		}
 	}
 	html_diag_destroy_cb(NULL, NULL, dg);
@@ -1385,10 +1462,8 @@ void quicklist_cb(GtkWidget * widget, gpointer data)
 
 	dg = html_diag_new(_("Quick List"));
 
-	dgtable = gtk_table_new(2, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
-
-	dg->spin[1] = spinbut_with_value(NULL, 0, 500, 1.0, 5.0);
+	dgtable = html_diag_table_in_vbox(dg, 2, 10);
+	dg->spin[1] = spinbut_with_value("3", 0, 500, 1.0, 5.0);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(dg->spin[1]), 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Rows")), 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[1], 1, 2, 0, 1);
@@ -1447,7 +1522,7 @@ static void framesetdialogok_lcb(GtkWidget * widget, Thtml_diag *dg) {
 	}
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("\n</FRAMESET>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("\n</FRAMESET>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -1468,8 +1543,7 @@ void framesetdialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Frameset"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Cols")), 0, 2, 0, 1);
@@ -1530,7 +1604,7 @@ static void framedialogok_lcb(GtkWidget * widget, Thtml_diag *dg) {
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -1554,8 +1628,7 @@ void framedialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Frame"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	dg->combo[1] = combo_with_popdown(tagvalues[0], main_v->current_project.urllist, 1);
 	file_but = file_but_new(GTK_WIDGET(GTK_COMBO(dg->combo[1])->entry), dg->dialog, 0);
@@ -1629,7 +1702,7 @@ static void embedok_lcb(GtkWidget * widget,Thtml_diag *dg )
 	}
 	g_free(thestring);
 
-	insert_dbl_text(finalstring, NULL);
+	doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	g_free(finalstring);
 	html_diag_destroy_cb(NULL, NULL, dg);
 }
@@ -1642,9 +1715,7 @@ void embed_cb(GtkWidget * widget, gpointer data)
 
 	dg = html_diag_new(_("Embed"));
 
-	dgtable = gtk_table_new(3, 12, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
-
+	dgtable = html_diag_table_in_vbox(dg, 3, 12);
 	dg->entry[1] = gtk_entry_new_with_max_length(256);
 	file_but = file_but_new(dg->entry[1], dg->dialog, 0);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(file_but), 10, 12, 0, 1);
@@ -1698,7 +1769,7 @@ static void script_linkok_lcb(gint type, GtkWidget * widget,Thtml_diag *dg ) {
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, endstring);
+		doc_insert_dbl_text(dg->doc, finalstring, endstring);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -1732,9 +1803,7 @@ static void script_link_cb(gint type, GtkWidget * widget, gpointer data) {
 		fill_dialogvalues(linkitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 	}
 
-	dgtable = gtk_table_new(3, 12, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
-
+	dgtable = html_diag_table_in_vbox(dg, 3, 12);
 	dg->entry[0] = entry_with_text(tagvalues[0], 1024);
 	file_but = file_but_new(dg->entry[0], dg->dialog, 0);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(file_but), 10, 12, 0, 1);
