@@ -20,7 +20,6 @@
  */
 #include "default_include.h"
 
-#include <locale.h>
 #include <string.h>  	/* strlen() */
 
 #include "gtk_easy.h"
@@ -1611,10 +1610,10 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	gtk_box_pack_start(GTK_BOX(hbox), scrolwin, FALSE, FALSE, 0);
 	
 	if (diag->styletype == multistyle) {
-		gchar *titles[] = {"Selector", "Property", "Value", NULL};
+		gchar *titles[] = {_("Selector"), _("Property"), _("Value"), NULL};
 		diag->clist = gtk_clist_new_with_titles(3, titles);
 	} else {
-		gchar *titles[] = {"Property", "Value", NULL};
+		gchar *titles[] = {_("Property"), _("Value"), NULL};
 		diag->clist = gtk_clist_new_with_titles(2, titles);
 	}
 	gtk_clist_set_sort_column(GTK_CLIST(diag->clist), 0);
@@ -1662,9 +1661,15 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 typedef enum {property, value, end} Tcs3_prevtype_onestyle;
 typedef enum {mselector, mproperty, mvalue, mend, mselectorend} Tcs3_prevtype_multistyle;
 
+/* static void css_parse(Tcs3_diag *diag, gchar *data)
+ * the buffer 'data' should contain a css stylesheet
+ * the comments are removed from this buffer by this function
+ * so if you don't want this, make a copy of the stylesheet
+ * before you pass it on to this function
+ */
 static void css_parse(Tcs3_diag *diag, gchar *data) {
 	gchar *tmp = data;
-	gint count = 0;
+	gint count = 0, parsable_css = 1;
 
 	if (!data) {
 		DEBUG_MSG("css_parse, no data!\n");
@@ -1678,7 +1683,7 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 		
 		DEBUG_MSG("css_parse, onestyle\n");
 		text[2] = NULL;
-		while (tmp[count]) {
+		while (tmp[count] && parsable_css) {
 			switch (tmp[count]) {
 			case ':':
 				if (prevtype == end) {
@@ -1696,6 +1701,28 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 					prevtype = value;
 				}
 			break;
+			case '*':
+				if ((count > 0)&& (tmp[count-1] == '/')) {
+					gint count2 = count, cont=1, movelen;
+					DEBUG_MSG("css_parse, found the start of a comment on %d\n", count);
+					while (tmp[++count2] && cont) {
+						if (tmp[count2] == '/' && tmp[count2-1] == '*') {
+							cont=0;
+						}
+					}
+					DEBUG_MSG("css_parse, before move, tmp='%s'\n", tmp);
+					movelen = strlen(&tmp[count2]);
+					memmove(&tmp[count-1], &tmp[count2], movelen);
+					tmp[count-1 + movelen] = '\0';
+					DEBUG_MSG("css_parse, after move, tmp='%s'\n", tmp);
+				}
+			break;
+			case '<':
+			case '>':
+				parsable_css = 0;
+				DEBUG_MSG("css_parse, this is not a valid stylesheet\n");
+			break;
+
 			default:
 				/* do nothing */
 			break;
@@ -1711,14 +1738,13 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 	
 	
 	} else { /* multistyle */
-		gint prev_end = 0;
+		gint prev_end = 0, i;
 		Tcs3_prevtype_multistyle prevtype = mselectorend;
-		gchar *text[4];
+		gchar *text[4] = {NULL, NULL, NULL, NULL};
 
 		DEBUG_MSG("css_parse, multistylestyle\n");		
-		text[3] = NULL;
 		
-		while (tmp[count]) {
+		while (tmp[count] && parsable_css) {
 			switch (tmp[count]) {
 			case '}': /* found end of block */
 				if (prevtype != mselectorend) {
@@ -1752,6 +1778,27 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 					prevtype = mvalue;
 				}
 			break;
+			case '*':
+				if ((count > 0)&& (tmp[count-1] == '/')) {
+					gint count2 = count, cont=1, movelen;
+					DEBUG_MSG("css_parse, found the start of a comment on %d\n", count);
+					while (tmp[++count2] && cont) {
+						if (tmp[count2] == '/' && tmp[count2-1] == '*') {
+							cont=0;
+						}
+					}
+					DEBUG_MSG("css_parse, before move, tmp='%s'\n", tmp);
+					movelen = strlen(&tmp[count2]);
+					memmove(&tmp[count-1], &tmp[count2], movelen);
+					tmp[count-1 + movelen] = '\0';
+					DEBUG_MSG("css_parse, after move, tmp='%s'\n", tmp);
+				}
+			break;
+			case '<':
+			case '>':
+				parsable_css = 0;
+				DEBUG_MSG("css_parse, this is not a valid stylesheet\n");
+			break;
 			default:
 				/* do nothing */
 			break;
@@ -1759,13 +1806,17 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 			if (prevtype == mvalue) {
 				gtk_clist_append(GTK_CLIST(diag->clist), text);
 				g_free(text[1]);
+				text[1] = NULL;
 				g_free(text[2]);
+				text[2] = NULL;
 				prevtype = mend;
 			}
 			count++;
 		}
-		if (text[0]){
-			g_free(text[0]);
+		for (i=0;i<4;i++) {
+			if (text[i]){
+				g_free(text[i]);
+			}
 		}
 	}
 	gtk_clist_sort(GTK_CLIST(diag->clist));
