@@ -21,7 +21,12 @@
 #ifndef __HTML2_H_
 #define __HTML2_H_
 void insert_class(GtkWidget * widget, gpointer data);
-void insert_div(GtkWidget * widget, gpointer data);
-void insert_span(GtkWidget * widget, gpointer data);
-void insert_in_stylesheet(GtkWidget * widget, gpointer data);
-#endif
+
+/* void insert_in_stylesheet(GtkWidget * widget, gpointer data); */
+
+#ifdef NEW_CS3
+GtkWidget *style_but_new(GtkWidget * which_entry, GtkWidget * win);
+void new_css_dialog(GtkWidget *widget, gpointer data);
+#endif /* NEW_CS3 */
+
+#endif /* __HTML2_H_ */
