@@ -23,7 +23,6 @@
 #include "default_include.h"
 
 #include <string.h>
-#include <locale.h>
 
 #include "cap.h"
 #include "bluefish.h"
@@ -106,7 +105,7 @@ static void formok_lcb(GtkWidget * widget, Thtml_diag *dg)
 
 	main_v->current_project.targetlist = add_entry_to_stringlist(main_v->current_project.targetlist, GTK_WIDGET(GTK_COMBO(dg->combo[3])->entry));
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</FORM>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</FORM>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -130,11 +129,21 @@ void formdialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Form"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(4, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 4, 10);
 
 	main_v->current_project.urllist = add_to_stringlist(main_v->current_project.urllist, "<?php echo $SCRIPT_NAME ?>");
-	dg->combo[4] = combo_with_popdown(tagvalues[0], main_v->current_project.urllist, 1);
+	main_v->current_project.urllist = add_to_stringlist(main_v->current_project.urllist, "'.$SCRIPT_NAME.'");
+	{
+	GList *rel_link_list, *tmplist;
+	rel_link_list = generate_relative_doc_list();
+	tmplist = duplicate_stringlist(main_v->current_project.urllist, 1);
+	rel_link_list = g_list_concat(tmplist, rel_link_list);
+
+	dg->combo[4] = combo_with_popdown(tagvalues[0], rel_link_list, 1);
+
+	free_stringlist(rel_link_list);
+	}
+	
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Action")), 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[4], 1, 8, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), file_but_new(GTK_WIDGET(GTK_COMBO(dg->combo[4])->entry), dg->dialog, 0), 8, 10, 0, 1);
@@ -185,7 +194,7 @@ static void textareaok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</TEXTAREA>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</TEXTAREA>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -205,8 +214,7 @@ void textareadialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Text area"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 0, 1);
@@ -247,7 +255,7 @@ static void textok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -268,8 +276,7 @@ void textdialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Text"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 0, 1);
@@ -317,7 +324,7 @@ static void buttondialogok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -336,8 +343,7 @@ void buttondialog_cb(GtkWidget * widget, gpointer data) {
 	dg = html_diag_new(_("Button"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 0, 1);
@@ -379,7 +385,7 @@ static void hiddenok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -400,9 +406,7 @@ void hiddendialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Hidden"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
-
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 10, 0, 1);
@@ -437,7 +441,7 @@ static void radiodialogok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -458,9 +462,7 @@ void radiodialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Radio Button"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
-
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 10, 0, 1);
@@ -492,9 +494,9 @@ static void checkdialogok_lcb(GtkWidget * widget,Thtml_diag *dg )
 
 	thestring = g_strdup(cap("<INPUT TYPE=\"CHECKBOX\""));
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[1])), cap("NAME"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[1])), cap("VALUE"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[2])), cap("VALUE"), thestring, NULL);
 	thestring = insert_attr_if_checkbox(dg->check[1], cap("CHECKED"), thestring);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[1])), NULL, thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[3])), NULL, thestring, NULL);
 	if (main_v->props.xhtml == 1) {
 		finalstring = g_strconcat(thestring," />", NULL);
 	} else {
@@ -503,7 +505,7 @@ static void checkdialogok_lcb(GtkWidget * widget,Thtml_diag *dg )
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, NULL);
+		doc_insert_dbl_text(dg->doc, finalstring, NULL);
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -522,9 +524,8 @@ void checkdialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Check Box"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
-
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
+	
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 10, 0, 1);
@@ -563,7 +564,7 @@ static void selectdialogok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</SELECT>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</SELECT>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -582,8 +583,7 @@ void selectdialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Select"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 0, 1);
@@ -619,7 +619,7 @@ static void optiondialogok_lcb(GtkWidget * widget,Thtml_diag *dg )
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</OPTION>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</OPTION>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -638,8 +638,7 @@ void optiondialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Option"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Value")), 0, 1, 0, 1);
@@ -669,7 +668,7 @@ static void optgroupdialogok_lcb(GtkWidget * widget,Thtml_diag *dg )
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</OPTGROUP>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</OPTGROUP>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -688,9 +687,7 @@ void optgroupdialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Option group"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 10, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
-
+	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 	dg->entry[1] = entry_with_text(tagvalues[0], 256);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Label")), 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 10, 0, 1);
