@@ -22,7 +22,6 @@
 #include "default_include.h"
 
 #include <string.h>
-#include <locale.h>
 
 #include "cap.h"  /* cap() */
 #include "bluefish.h"
@@ -59,7 +58,7 @@ static void tabledialogok_lcb(GtkWidget * widget, Thtml_diag * dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</TABLE>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</TABLE>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -84,8 +83,7 @@ void tabledialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Table"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 8, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 8);
 
 	dg->spin[1] = spinbut_with_value(tagvalues[0], 0, 100, 1.0, 5.0);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Cell Padding")), 0, 1, 0, 1);
@@ -117,7 +115,17 @@ void tabledialog_cb(GtkWidget * widget, gpointer data)
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Align")), 2, 3, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[1], 3, 5, 0, 1);
 
-	dg->combo[2] = combo_with_popdown_sized(tagvalues[4], positionlist, 1, 80);
+  alignlist = NULL;
+	alignlist = g_list_insert(alignlist, "top", 0);
+	alignlist = g_list_insert(alignlist, "middle", 1);
+	alignlist = g_list_insert(alignlist, "bottom", 2);
+	alignlist = g_list_insert(alignlist, "texttop", 3);
+	alignlist = g_list_insert(alignlist, "baseline", 4);
+	alignlist = g_list_insert(alignlist, "absmiddle", 5);
+	alignlist = g_list_insert(alignlist, "absbottom", 6);
+	dg->combo[2] = combo_with_popdown_sized(tagvalues[4], alignlist, 1, 80);
+	g_list_free(alignlist);
+
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Vertical Align")), 2, 3, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[2], 3, 5, 1, 2);
 
@@ -185,7 +193,7 @@ static void tablerowdialogok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	g_free(thestring);
 
 	if (dg->range.len == -1) {
-		insert_dbl_text(finalstring, cap("</TR>"));
+		doc_insert_dbl_text(dg->doc, finalstring, cap("</TR>"));
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
 	}
@@ -208,8 +216,7 @@ void tablerowdialog_cb(GtkWidget * widget, gpointer data)
 	dg = html_diag_new(_("Table Row"));
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(4, 5, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 4, 5);
 
 	alignlist = g_list_insert(alignlist, "left", 0);
 	alignlist = g_list_insert(alignlist, "right", 1);
@@ -219,7 +226,17 @@ void tablerowdialog_cb(GtkWidget * widget, gpointer data)
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Align")), 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(dg->combo[1]), 1, 2, 0, 1);
 
-	dg->combo[2] = combo_with_popdown_sized(tagvalues[1], positionlist, 0, 90);
+  alignlist = NULL;
+	alignlist = g_list_insert(alignlist, "top", 0);
+	alignlist = g_list_insert(alignlist, "middle", 1);
+	alignlist = g_list_insert(alignlist, "bottom", 2);
+	alignlist = g_list_insert(alignlist, "texttop", 3);
+	alignlist = g_list_insert(alignlist, "baseline", 4);
+	alignlist = g_list_insert(alignlist, "absmiddle", 5);
+	alignlist = g_list_insert(alignlist, "absbottom", 6);
+	dg->combo[2] = combo_with_popdown_sized(tagvalues[1], alignlist, 0, 90);
+	g_list_free(alignlist);
+
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("VAlign")), 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[2])), 1, 2, 1, 2);
 
@@ -259,7 +276,7 @@ static void table_head_and_data_dialogok_lcb(gint type, GtkWidget * widget, Thtm
 	}
 	thestring = insert_integer_if_spin(dg->spin[1], cap("WIDTH"), thestring, dg->check[2]);
 	thestring = insert_integer_if_spin(dg->spin[3], cap("HEIGHT"), thestring, dg->check[3]);
-	thestring = insert_string_if_entry(dg->spin[1], cap("COLSPAN"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->spin[5], cap("COLSPAN"), thestring, NULL);
 	thestring = insert_string_if_entry(dg->spin[4], cap("ROWSPAN"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[1])->entry), cap("ALIGN"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[2])->entry), cap("VALIGN"), thestring, NULL);
@@ -273,9 +290,9 @@ static void table_head_and_data_dialogok_lcb(gint type, GtkWidget * widget, Thtm
 
 	if (dg->range.len == -1) {
 		if (type == 1) {
-			insert_dbl_text(finalstring, cap("</TD>"));
+			doc_insert_dbl_text(dg->doc, finalstring, cap("</TD>"));
 		} else {
-			insert_dbl_text(finalstring, cap("</TH>"));
+			doc_insert_dbl_text(dg->doc, finalstring, cap("</TH>"));
 		}
 	} else {
 		replace_text(finalstring, dg->range.pos, dg->range.len);
@@ -312,8 +329,7 @@ static void table_head_and_data_dialog_cb(gint type, GtkWidget * widget, gpointe
 	
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, widget, dg);
 
-	dgtable = gtk_table_new(5, 7, 0);
-	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
+	dgtable = html_diag_table_in_vbox(dg, 5, 7);
 
 	alignlist = g_list_insert(alignlist, "left", 0);
 	alignlist = g_list_insert(alignlist, "right", 1);
@@ -324,7 +340,17 @@ static void table_head_and_data_dialog_cb(gint type, GtkWidget * widget, gpointe
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Align")), 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[1])), 1, 2, 0, 1);
 
-	dg->combo[2] = combo_with_popdown_sized(tagvalues[4], positionlist, 1, 80);
+  alignlist = NULL;
+	alignlist = g_list_insert(alignlist, "top", 0);
+	alignlist = g_list_insert(alignlist, "middle", 1);
+	alignlist = g_list_insert(alignlist, "bottom", 2);
+	alignlist = g_list_insert(alignlist, "texttop", 3);
+	alignlist = g_list_insert(alignlist, "baseline", 4);
+	alignlist = g_list_insert(alignlist, "absmiddle", 5);
+	alignlist = g_list_insert(alignlist, "absbottom", 6);
+	dg->combo[2] = combo_with_popdown_sized(tagvalues[4], alignlist, 1, 90);
+	g_list_free(alignlist);
+
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("VAlign")), 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[2])), 1, 2, 1, 2);
 
@@ -342,9 +368,9 @@ static void table_head_and_data_dialog_cb(gint type, GtkWidget * widget, gpointe
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Custom")), 0, 1, 4, 5);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 7, 4, 5);
 
-	dg->spin[1] = spinbut_with_value(tagvalues[2], 0, 10, 1.0, 2.0);
+	dg->spin[5] = spinbut_with_value(tagvalues[2], 0, 10, 1.0, 2.0);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Col Span")), 2, 3, 0, 1);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[1], 3, 4, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[5], 3, 4, 0, 1);
 
 	dg->spin[4] = spinbut_with_value(tagvalues[5], 0, 100, 1.0, 5.0);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Row Span")), 2, 3, 1, 2);
