@@ -17,6 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+/* #define IMAGEMAP_EDITOR_TEST */
+
 
 #ifndef __IMAGES_H_
 #define __IMAGES_H_
@@ -25,6 +27,11 @@
 #include <gtk/gtkwidget.h>
 
 void image_insert_dialog_cb(GtkWidget * widget, gpointer data);
-void image_thumbnail_dialog_cb(GtkWidget * widget, gpointer data);
 
+#ifdef HAVE_IMAGE_LIBS
+void image_thumbnail_dialog_cb(GtkWidget * widget, gpointer data);
+void preview_image_by_filename (char *filename);
+void preview_image_by_position_cb (GtkWidget * widget, gpointer data);
+
+#endif
 #endif							/* __IMAGES_H_ */
