@@ -3,6 +3,7 @@
  *
  * Copyright (C) 1998-1999 Olivier Sessink and Chris Mazuc
  * rcfile code Copyright (C) 1999 MJ Ray and Hylke van der Schaaf
+ * Copyright (C) 2000-2001 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,10 +29,15 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include <locale.h>
+#include "config.h"
+
+#ifdef HAVE_GETOPT_H /* Solaris doesn't have it, it's in stdlib.h*/
+#include <getopt.h>
+#endif
 
 #include "bluefish.h"
 #include "bf_lib.h"
+#include "gtk_easy.h"
 #include "init.h"
 #include "stringlist.h"
 #include "filter.h"
@@ -91,6 +97,7 @@ static gint chk_dir(gchar * name)
  */
 static gint chk_link(gchar * src_name, gchar * dst_name)
 {
+#ifndef WIN32
 	struct stat filestat;
 
 	if ((stat(src_name, &filestat) == -1) && (errno == ENOENT)) {
@@ -105,6 +112,8 @@ static gint chk_link(gchar * src_name, gchar * dst_name)
 		};
 	};
 	return 1;
+#endif
+	return 0;
 };
 
 
@@ -166,8 +175,9 @@ gint check_files_links_directories()
 	/* another g_get_home_dir () */
 	homedir = g_get_home_dir();
 	DEBUG_MSG("check_directories, homedir=%s\n", homedir);
-	chk_dir(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/", PATH_MAX));
-	chk_dir(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/projects/", PATH_MAX));
+	/* mkdir fails on openBSD if there is a trailing slash */
+	chk_dir(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish", PATH_MAX));
+	chk_dir(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/projects", PATH_MAX));
 	chk_link(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/php3_functions", PATH_MAX),
 			 strncat(strncpy(name2, PKGDATADIR, PATH_MAX), "/php3_functions", PATH_MAX));
 	chk_link(strncat(strncpy(name, homedir, PATH_MAX), "/.bluefish/ssi_functions", PATH_MAX),
@@ -369,7 +379,7 @@ gint save_config_file(GList * config_list, gchar * filename)
 		DEBUG_MSG("save_config_file, datatype %c\n", tmpitem->type);
 		switch (tmpitem->type) {
 		case 'i':
-			DEBUG_MSG("save_config_file, converting \"%p\"\n", tmpitem);
+			DEBUG_MSG("save_config_file, converting \"%p\" to integer\n", tmpitem);
 			DEBUG_MSG("save_config_file, converting \"%s %i\"\n", tmpitem->identifier, *(int *) (void *) tmpitem->pointer);
 
 			tmpstring = g_strdup_printf("%s %i", tmpitem->identifier, *(int *) (void *) tmpitem->pointer);
@@ -380,7 +390,7 @@ gint save_config_file(GList * config_list, gchar * filename)
 			rclist = g_list_append(rclist, tmpstring);
 			break;
 		case 's':
-			DEBUG_MSG("save_config_file, converting \"%p\"\n", tmpitem);
+			DEBUG_MSG("save_config_file, converting \"%p\" to string\n", tmpitem);
 			DEBUG_MSG("save_config_file, converting \"%s %s\"\n", tmpitem->identifier, (gchar *) * (void **) tmpitem->pointer);
 			if (*(void **) tmpitem->pointer) {
 				tmpstring = g_strdup_printf("%s %s", tmpitem->identifier, (gchar *) * (void **) tmpitem->pointer);
@@ -504,7 +514,6 @@ gint parse_config_file(GList * config_list, gchar * filename)
 					switch (tmpitem->type) {
 					case 'i':
 						*(int *) (void *) tmpitem->pointer = atoi(tmpstring);
-
 						break;
 					case 's':
 						(char *) *(void **) tmpitem->pointer = realloc((char *) *(void **) tmpitem->pointer, strlen(tmpstring) + 1);
@@ -583,15 +592,61 @@ static void external_filter_lcb(GtkWidget * widget, gpointer data)
 	g_free(in.buf);
 }
 
+/* static gchar *command_or_filter_replace(gchar *commandstring)
+ * returns a newly allocated string with the following replaces:
+ * %s -> current filename
+ * (for filters, not yet implemented --> %o -> output filename that should be read into bluefish)
+ * %n -> new filename that should be asked from the user (export)
+ */
+static gchar *command_or_filter_replace(gchar *commandstring) {
+	gchar *newstr= g_strdup(commandstring), *tmpstr;
+	gint len = strlen(commandstring), i;
+	
+	for (i=0;i<len;i++) {
+		if (newstr[i] == '%' && i < (len-1)) {
+			gchar *replacestr=NULL;
+			switch (newstr[i+1]) {
+			case 's':
+				if (main_v->current_document->filename) {
+					replacestr = g_strdup(main_v->current_document->filename);
+				}
+			break;
+/*			case 'o':
+			
+			break; */
+			case 'n':
+				replacestr = return_file_w_title(NULL, _("Export filename"));
+			break;
+			}
+			if (replacestr) {
+				gint offset = strlen(replacestr) -2;
+				newstr[i] = '\0';
+				tmpstr = g_strconcat(newstr, replacestr, &newstr[i+2], NULL);
+				g_free(newstr);
+				newstr = tmpstr;
+				i += offset;
+				len += offset;
+				DEBUG_MSG("command_or_filter_replace, newstr=%s\n", newstr);
+			}
+		
+		}
+	
+	}
+	return newstr;
+}
 
 static void external_command_lcb(GtkWidget * widget, gpointer data)
 {
 	Texternal_menu *exmen;
+	gchar *newcommand;
 
 	DEBUG_MSG("external_command_cb, started with %d\n", GPOINTER_TO_INT(data));
 	exmen = (Texternal_menu *) g_list_nth_data(main_v->command_strings, GPOINTER_TO_INT(data));
 	DEBUG_MSG("external_command_cb, command=%s\n", exmen->command);
-	system(exmen->command);
+	newcommand = command_or_filter_replace(exmen->command);
+	DEBUG_MSG("external_command_cb, newcommand=%s\n", newcommand);
+	system(newcommand);
+	g_free(newcommand);
 	DEBUG_MSG("external_command_cb, finished\n");
 }
 
@@ -677,8 +732,10 @@ static GList *main_v_props_init(GList * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.v_html_tb, "view_html_toolbar:", 1);
 	init_prop_integer(&config_rc, &main_v->props.v_custom_tb, "view_custom_toolbar:", 1);
 	init_prop_integer(&config_rc, &main_v->props.v_main_tb, "view_main_toolbar:", 1);
+	init_prop_integer(&config_rc, &main_v->props.v_filebrowser, "view_filebrowser:", 1);
 	init_prop_integer(&config_rc, &main_v->props.transient_htdialogs, "transient_htdialogs:", 0);
 	init_prop_string(&config_rc, &main_v->props.cfg_browser_cline, "browser_command:", NETSCAPE_COMMAND);
+	init_prop_string(&config_rc, &main_v->props.image_editor_cline, "image_editor_command:", "gimp-remote -n \"%s\"&");
 	init_prop_string(&config_rc, &main_v->props.cfg_editor_font, "editor_font:", "fixed,*");
 	init_prop_integer(&config_rc, &main_v->props.cfg_editor_tabwidth, "editor_tab_width:", 8);
 	init_prop_string(&config_rc, &main_v->props.cfg_tab_font, "notebook_tab_font:", "fixed,*");
@@ -734,12 +791,17 @@ static GList *main_v_props_init(GList * config_rc)
 	init_prop_integer (&config_rc, &main_v->props.spc_accept_compound,"spell_checker_accept_compound:",0);
 	init_prop_integer (&config_rc, &main_v->props.spc_use_esc_chars,"spell_checker_use_esc_chars:",0);
 	init_prop_string  (&config_rc, &main_v->props.spc_esc_chars,"spell_checker_esc_chars:",NULL);
-	init_prop_integer (&config_rc, &main_v->props.spc_use_pers_dict,"spell_checker_use,pers_dict:",0);
-	init_prop_string  (&config_rc, &main_v->props.spc_use_pers_dict,"spell_checker_pers_dict:",NULL);
+	init_prop_integer (&config_rc, &main_v->props.spc_use_pers_dict,"spell_checker_use_pers_dict:",0);
+	init_prop_string  (&config_rc, &main_v->props.spc_pers_dict,"spell_checker_pers_dict:",NULL);
 #endif
 	init_prop_string  (&config_rc, &main_v->props.backup_filestring,"backup_filestring:","~");
 	init_prop_integer (&config_rc, &main_v->props.backup_file,"backup_file:",1);
+	init_prop_integer (&config_rc, &main_v->props.backup_by_copy,"backup_by_copy:",0);
+	init_prop_string (&config_rc, &main_v->props.backup_abort_style,"backup_abort_style:","ask");
+	init_prop_integer (&config_rc, &main_v->props.auto_convert_CR,"auto_convert_CR:",1);
 	init_prop_integer (&config_rc, &main_v->props.allow_multi_instances,"allow_multi_instances:",0);
+	init_prop_integer (&config_rc, &main_v->props.num_undo_levels,"num_undo_levels:",100);
+	init_prop_integer (&config_rc, &main_v->props.clear_undo_on_save,"clear_undo_on_save:",0);
 
 	/* key conversion */
 	init_prop_integer (&config_rc, &main_v->props.conv_ctrl_enter,"conv_ctrl_enter:",0);
@@ -750,6 +812,11 @@ static GList *main_v_props_init(GList * config_rc)
 #ifdef WITH_MSG_QUEUE
 	init_prop_integer (&config_rc, &main_v->props.open_in_running_bluefish,"open_in_running_bluefish:",1);
 #endif
+#ifdef AUTOCOMPLET
+	init_prop_integer (&config_rc, &main_v->props.autocomplet_html,"autocomplet_html_tags:",0);
+	init_prop_integer (&config_rc, &main_v->props.autocomplet_delay,"autocomplet_delay:",0);
+#endif	
+
 	return config_rc;
 }
 
@@ -799,7 +866,11 @@ void save_config_files(void)
 	DEBUG_MSG("save_config_files, started\n");
 
 	/* g_get_home_dir() is still broken !!!!! don't change this into the glib functions!!! */
+#ifndef WIN32
 	filename = g_strconcat(getenv("HOME"), "/.bluefish/rcfile", NULL);
+#else
+	filename = g_strconcat("C:\\", "rcfile", NULL);
+#endif
 	CHECK_MEMORY(filename == NULL, "save_config_file:filename");
 	save_config_file(config_rc, filename);
 	g_free(filename);
