@@ -2,7 +2,7 @@
  * interface.c - this file contains the UI code and some init code
  *
  * Copyright (C) 1998-1999 Olivier Sessink and Chris Mazuc
- * Copyright (C) 1999-2000 Olivier Sessink 
+ * Copyright (C) 1999-2001 Olivier Sessink 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,8 +27,11 @@
 #include <dirent.h>  /* opendir() */
 #include <sys/stat.h> /* stat() */
 #include <unistd.h>  /* stat() */
-#include <locale.h>
 #include <stdio.h> /* rename() */
+#include <errno.h>			/* errno */
+#include <time.h> /* ctime() */
+#include <pwd.h> /* getpwuid() */
+#include <grp.h> /* getgrgid() */
 
 #include "bluefish.h" /* identify_by_filename() */
 #include "bf_lib.h"
@@ -36,8 +39,13 @@
 #include "interface.h"
 #include "highlight.h"  /* make syntax struct list */
 #include "init.h"
+#ifdef UNDO2
+#include "undo2.h"
+#else
 #include "undo.h"
+#endif /* UNDO2 */
 #include "toolbars.h"
+#include "images.h" /*preview_image_by_filename */
 #include "callbacks.h"
 #include "rpopup.h"
 #include "document.h"
@@ -49,6 +57,14 @@
 #include "../pixmaps/bluefish_icon.xpm"
 #include "../pixmaps/bluefish.xpm"
 
+#ifdef WIN32
+#define DIRSTR "\\"
+#define DIRCHR 92
+#else
+#define DIRSTR "/"
+#define DIRCHR '/'
+#endif
+
 typedef struct {
 	GtkWidget *cust_handle_box;
 	GtkWidget *handle_box;
@@ -76,6 +92,8 @@ static const GtkTargetEntry drag_dest_types[] = {
 static gint context_id = 0;
 static Thidegui hidewidgets;
 
+static char str[11];
+
 /********************************************************************/
 
 static void hide_gui(gint *configvar, GtkWidget *widget) {
@@ -91,6 +109,16 @@ static void show_gui(gint *configvar, GtkWidget **widget, void func()) {
 	gtk_widget_show(*widget);
 }
 
+void toggle_filebrowser_cb(GtkWidget * w, gpointer data) {
+	if (main_v->filebrowser.hpane) {
+		filebrowser_hide(0);
+		main_v->props.v_filebrowser = 0;
+	} else {
+		filebrowser_show(0);
+		main_v->props.v_filebrowser = 1;
+	}
+}
+
 void toggle_cust_menubar_cb(GtkWidget * w, gpointer data) {
 	if (GTK_WIDGET_VISIBLE(hidewidgets.cust_handle_box)) {
 		hide_gui(&main_v->props.v_custom_tb, hidewidgets.cust_handle_box);
@@ -130,7 +158,7 @@ static void add_to_dir_history(gchar *dirname);
 void file_and_dir_history_add(gchar *filename) {
 	gchar *dirname;
 
-	add_to_recent_list(filename); /* the recent menu */
+	add_to_recent_list(filename, 0); /* the recent menu */
 	dirname = g_dirname(filename);
 	add_to_dir_history(dirname);
 	g_free(dirname);
@@ -149,12 +177,18 @@ gint statusbar_remove(gpointer message_id)
 
 void statusbar_message(gpointer message, gint time)
 {
+	if (main_v->statusbar) {
+		gint count;
 
-	gint count;
-
-	count = gtk_statusbar_push(GTK_STATUSBAR(main_v->statusbar), context_id, (gchar *) message);
-	gtk_timeout_add(time, statusbar_remove, GINT_TO_POINTER(count));
-	flush_queue();
+		count = gtk_statusbar_push(GTK_STATUSBAR(main_v->statusbar), context_id, (gchar *) message);
+		gtk_timeout_add(time, statusbar_remove, GINT_TO_POINTER(count));
+		flush_queue();
+	}
+#ifdef DEVELOPMENT
+	else {
+		g_print("statusbar_message: %s\n", (gchar *)message);
+	}
+#endif
 }
 
 gint bf_statusbar_message(gpointer message)
@@ -219,12 +253,11 @@ void notebook_changed(void)
 			DEBUG_MSG("notebook_changed, it did really change to a new document, cur=%d\n", cur);
 			main_v->last_notebook_page = cur;
 			main_v->current_document = g_list_nth_data(main_v->documentlist, cur);
-#ifdef DEBUG
+#ifdef DEVELOPMENT
 			DEBUG_MSG("notebook_changed, finished, main_v->current_document=%p\n", main_v->current_document);			
 			g_assert(main_v->current_document);
 #endif
-			DEBUG_MSG("notebook_changed, grabbing document %p textbox=%p\n", main_v->current_document, main_v->current_document->textbox);
-			gtk_widget_grab_focus(GTK_WIDGET(main_v->current_document->textbox));
+			notebook_switched_to_doc(main_v->current_document);
 			DEBUG_MSG("notebook_changed, setting up toggle item\n");
 			setup_toggle_item(gtk_item_factory_from_widget(main_v->menubar), N_("/View/Highlight syntax"),
 							  main_v->current_document->highlightstate);
@@ -369,23 +402,23 @@ static void on_drag_data_cb(GtkWidget * widget, GdkDragContext * context
 	} else if (type == TYPE_IMAGE) {
 		DEBUG_MSG("on_drag_data_cb, ends in .gif,.jpg,.jpeg,.png\n");
 		if (rel_filename) {
-			tmp = g_strconcat("<img src=\"", rel_filename, "\" border=\"0\" alt=\"\">", NULL);
+			tmp = g_strconcat("<img src=\"", rel_filename, "\" border=\"0\" alt=\"\"", main_v->props.xhtml == 1 ? " />" : ">", NULL);
 		} else {
-			tmp = g_strconcat("<img src=\"", url, "\" border=\"0\" alt=\"\">", NULL);
+			tmp = g_strconcat("<img src=\"", url, "\" border=\"0\" alt=\"\"", main_v->props.xhtml == 1 ? " />" : ">", NULL);
 		}
 	} else if (type == TYPE_CSS) {
 		DEBUG_MSG("on_drag_data_cb, type is TYPE_CSS\n");
 		if (rel_filename) {
-			tmp = g_strconcat("<link rel=\"stylesheet\" href=\"", rel_filename, "\" type=\"text/css\">", NULL);
+			tmp = g_strconcat("<link rel=\"stylesheet\" href=\"", rel_filename, "\" type=\"text/css\"", main_v->props.xhtml == 1 ? " />" : ">", NULL);
 		} else {
-			tmp = g_strconcat("<link rel=\"stylesheet\" href=\"", url, "\" type=\"text/css\">", NULL);
+			tmp = g_strconcat("<link rel=\"stylesheet\" href=\"", url, "\" type=\"text/css\"", main_v->props.xhtml == 1 ? " />" : ">", NULL);
 		}
 		DEBUG_MSG("on_drag_data_cb, string=%s\n", tmp);
 	} else if (type == TYPE_JSCRIPT) {
 		if (rel_filename) {
-			tmp = g_strconcat("<script language=\"javascript\" src=\"", rel_filename, "\"></script>", NULL);
+			tmp = g_strconcat("<script language=\"javascript\" type=\"text/javascript\" src=\"", rel_filename, "\"></script>", NULL);
 		} else {
-			tmp = g_strconcat("<script language=\"javascript\" src=\"", url, "\"></script>", NULL);
+			tmp = g_strconcat("<script language=\"javascript\" type=\"text/javascript\" src=\"", url, "\"></script>", NULL);
 		}
 	} else {
 		DEBUG_MSG("on_drag_data_cb, unknown file format\n");
@@ -416,8 +449,94 @@ static void on_drag_data_cb(GtkWidget * widget, GdkDragContext * context
 
 }
 
+typedef struct {
+	GtkWidget *win;
+	GtkWidget *entry;
+	gint is_file;
+} Tcfod;
+
 /* declaration needed here */
 static void browse_to_dir(gchar *dirname);
+static void file_list_rpopup_refresh(GtkWidget *widget, gpointer data);
+
+static void create_file_or_dir_destroy_lcb(GtkWidget *widget, GdkEvent *event,Tcfod *ws) {
+	window_destroy(ws->win);
+	g_free(ws);
+}
+
+static void create_file_or_dir_cancel_clicked_lcb(GtkWidget *widget, Tcfod *ws) {
+	create_file_or_dir_destroy_lcb(NULL, NULL, ws);
+}
+
+static void create_file_or_dir_ok_clicked_lcb(GtkWidget *widget, Tcfod *ws) {
+	gchar *newname, *name;
+
+	name = gtk_editable_get_chars(GTK_EDITABLE(ws->entry), 0, -1);	
+	if (name) {
+		if (strlen(name)) {
+			newname = g_strconcat(main_v->filebrowser.curdir,DIRSTR,name, NULL);
+			DEBUG_MSG("create_file_or_dir_ok_clicked_lcb, newname=%s\n", newname);
+			if (file_exists_and_readable(newname)) {
+				error_dialog(_("Error creating path"),_("Path exists already"));
+			} else {
+				if (ws->is_file) {
+					doc_new_with_new_file(newname);
+				} else {
+					if(mkdir(newname, 0755)== -1) {
+						error_dialog(_("Error creating directory"),strerror(errno));
+					}
+				}
+			}
+			g_free(newname);
+			file_list_rpopup_refresh(NULL, NULL);
+		}
+		g_free(name);
+	}
+	create_file_or_dir_destroy_lcb(NULL, NULL, ws);
+}
+
+static void create_file_or_dir_win(gint is_file) {
+	Tcfod *ws;
+	gchar *title;
+	GtkWidget *vbox, *but, *hbox;
+
+	ws = g_malloc(sizeof(Tcfod));
+	if (is_file) {
+		title = _("File name");
+	} else {
+		title = _("Directory name");
+	}
+	ws->is_file = is_file;
+	
+	ws->win = window_full(title, GTK_WIN_POS_MOUSE, GTK_WINDOW_DIALOG
+	, 5,create_file_or_dir_destroy_lcb , ws);
+	vbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(ws->win), vbox);
+	ws->entry = boxed_entry_with_text(NULL, 250, vbox);
+	
+	hbox = gtk_hbutton_box_new();
+	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 1);
+	
+	but = bf_stock_ok_button(create_file_or_dir_ok_clicked_lcb, ws);
+	gtk_box_pack_start(GTK_BOX(hbox),but , FALSE, FALSE, 0);
+	
+	but = bf_stock_cancel_button(create_file_or_dir_cancel_clicked_lcb, ws);
+	gtk_box_pack_start(GTK_BOX(hbox),but , FALSE, FALSE, 0);
+	
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	
+	gtk_widget_grab_focus(ws->entry);
+	gtk_widget_show_all(GTK_WIDGET(ws->win));
+}
+
+static void create_file_lcb(GtkWidget *widget, gpointer data) {
+	create_file_or_dir_win(1);
+}
+
+static void create_directory_lcb(GtkWidget *widget, gpointer data) {
+	create_file_or_dir_win(0);
+}
 
 static void dir_history_activate_lcb(GtkWidget *widget, gpointer data) {
 
@@ -459,12 +578,12 @@ static void add_to_dir_history(gchar *dirname) {
 		return;
 	}
 	DEBUG_MSG("add_to_dir_history, checking in %s\n", dirname);	
-	if (dirname[strlen(dirname)] != '/') {
-		tmpstr = g_strconcat(dirname, "/", NULL);
+	if (dirname[strlen(dirname)] != DIRCHR) {
+		tmpstr = g_strconcat(dirname, DIRSTR, NULL);
 	} else {
 		tmpstr = g_strdup(dirname);
 	}
-	
+
 	main_v->filebrowser.directory_history = add_to_stringlist(main_v->filebrowser.directory_history, tmpstr);
 	g_free(tmpstr);
 
@@ -496,9 +615,9 @@ void populate_dir_history() {
 		main_v->filebrowser.curdir = g_strdup(g_get_current_dir());
 	}
 
-	tmpdir = g_strconcat(main_v->filebrowser.curdir, "/", NULL);
+	tmpdir = g_strconcat(main_v->filebrowser.curdir, DIRSTR, NULL);
 	new_history_list = add_to_stringlist(new_history_list, tmpdir);
-	while ((tmpchar = strrchr(tmpdir, '/'))) {
+	while ((tmpchar = strrchr(tmpdir, DIRCHR))) {
 		tmpchar++;
 		*tmpchar = '\0';
 		DEBUG_MSG("populate_dir_history, adding %s\n", tmpdir);
@@ -508,18 +627,18 @@ void populate_dir_history() {
 	}
 	g_free(tmpdir);
 	
-	tmpdir = g_strconcat(g_get_current_dir(), "/", NULL);
+	tmpdir = g_strconcat(g_get_current_dir(), DIRSTR, NULL);
 	new_history_list = add_to_stringlist(new_history_list, tmpdir);
 	g_free(tmpdir);
 
-	tmpdir = g_strconcat(g_get_home_dir(), "/", NULL);
+	tmpdir = g_strconcat(g_get_home_dir(), DIRSTR, NULL);
 	new_history_list = add_to_stringlist(new_history_list, tmpdir);
 	g_free(tmpdir);
 
-	tmplist = g_list_first(main_v->filebrowser.directory_history);
+	tmplist = g_list_last(main_v->filebrowser.directory_history);
 	while (tmplist) {
 		new_history_list = add_to_stringlist(new_history_list, (gchar *)tmplist->data);
-		tmplist = g_list_next(tmplist);
+		tmplist = g_list_previous(tmplist);
 	} 
 
 	/* make the actual option menu */
@@ -611,7 +730,7 @@ void populate_dir_file_list() {
 			if (S_ISDIR(statbuf.st_mode)) {
 				gchar *tmpstr;
 
-				tmpstr = g_strconcat(ep->d_name, "/", NULL);
+				tmpstr = g_strconcat(ep->d_name, DIRSTR, NULL);
 				text[0] = tmpstr;
 				row = gtk_clist_append(GTK_CLIST(main_v->filebrowser.dir_list), text);
 				gtk_clist_set_pixtext(GTK_CLIST(main_v->filebrowser.dir_list)
@@ -646,6 +765,10 @@ void populate_dir_file_list() {
 	gtk_clist_thaw(GTK_CLIST(main_v->filebrowser.file_list));
 }
 
+static void file_list_rpopup_refresh(GtkWidget *widget, gpointer data) {
+	populate_dir_file_list();
+}
+
 static void dir_list_activate_row(gint row) {
 	gchar *temp=NULL;
 	guint8 spacing;
@@ -654,7 +777,7 @@ static void dir_list_activate_row(gint row) {
 
 	gtk_clist_get_pixtext (GTK_CLIST (main_v->filebrowser.dir_list), row, 0, &temp, &spacing, &pixmap, &bitmap);
 	if (temp) {
-		gchar *dirname = g_strconcat(main_v->filebrowser.curdir, "/", temp, NULL);
+		gchar *dirname = g_strconcat(main_v->filebrowser.curdir, DIRSTR, temp, NULL);
 		DEBUG_MSG("dir_list_activate_row, change to %s\n", dirname);
 		browse_to_dir(dirname);
 		g_free (dirname);
@@ -670,10 +793,6 @@ static void dir_list_select_lcb(GtkWidget *widget, gint row, gint column, GdkEve
 	}
 }
 
-static void file_list_rpopup_refresh(GtkWidget *widget, gpointer data) {
-	populate_dir_file_list();
-}
-
 static gchar *filebrowser_get_filename_from_row(gint row) {
 	if (row != -1) {
 		gchar *temp=NULL;
@@ -684,7 +803,7 @@ static gchar *filebrowser_get_filename_from_row(gint row) {
 		gtk_clist_get_pixtext (GTK_CLIST (main_v->filebrowser.file_list), row, 0, &temp, &spacing, &pixmap, &bitmap);
 		if (temp && strlen(temp)) {
 			DEBUG_MSG("filebrowser_get_filename_from_row, found filename %s\n", temp);
-			return g_strconcat(main_v->filebrowser.curdir, "/", temp, NULL);
+			return g_strconcat(main_v->filebrowser.curdir, DIRSTR, temp, NULL);
 		}
 #ifdef DEBUG
 		else {
@@ -709,6 +828,27 @@ static void filebrowser_file_open_lcb(GtkWidget *widget, gpointer data) {
 		g_free(filename);
 	}
 }
+#ifdef HAVE_IMAGE_LIBS
+static void filebrowser_preview_image_lcb(GtkWidget *widget, gpointer data) {
+	gchar *filename;
+	
+	filename = filebrowser_get_filename_from_row(main_v->filebrowser.file_list_selectedrow);
+	if (filename) {
+		DEBUG_MSG("filebrowser_preview_image_lcb, previewing %s\n", filename);
+		preview_image_by_filename (filename);
+		g_free(filename);
+	}
+}
+#endif /* HAVE_IMAGE_LIBS */
+static void filebrowser_edit_image_lcb(GtkWidget *widget, gpointer data) {
+	gchar *filename;
+	
+	filename = filebrowser_get_filename_from_row(main_v->filebrowser.file_list_selectedrow);
+	if (filename) {
+		open_image_in_editor(filename);
+		g_free(filename);
+	}
+}
 
 static void filebrowser_file_delete_lcb(GtkWidget *widget, gpointer data) {
 	gchar *filename;
@@ -730,6 +870,125 @@ static void filebrowser_file_delete_lcb(GtkWidget *widget, gpointer data) {
 	}
 }
 
+static void rwx (short unsigned int bits, char *chars)
+{
+  chars[0] = (bits & S_IRUSR) ? 'r' : '-';
+  chars[1] = (bits & S_IWUSR) ? 'w' : '-';
+  chars[2] = (bits & S_IXUSR) ? 'x' : '-';
+}
+
+static void setst (short unsigned int bits, char *chars)
+{
+#ifdef S_ISUID
+  if (bits & S_ISUID)
+    {
+      if (chars[3] != 'x')
+	/* Set-uid, but not executable by owner.  */
+	chars[3] = 'S';
+      else
+	chars[3] = 's';
+    }
+#endif
+#ifdef S_ISGID
+  if (bits & S_ISGID)
+    {
+      if (chars[6] != 'x')
+	/* Set-gid, but not executable by group.  */
+	chars[6] = 'S';
+      else
+	chars[6] = 's';
+    }
+#endif
+#ifdef S_ISVTX
+  if (bits & S_ISVTX)
+    {
+      if (chars[9] != 'x')
+	/* Sticky, but not executable by others.  */
+	chars[9] = 'T';
+      else
+	chars[9] = 't';
+    }
+#endif
+}
+
+static void filemode_to_string(mode_t statmode) {
+ 	/* following code "adapted" from GNU filemode.c program */
+
+str[0]='?';
+#ifdef S_ISBLK
+  if (S_ISBLK (statmode))
+    str[0]='b';
+#endif
+  if (S_ISCHR (statmode))
+    str[0]='c';
+  if (S_ISDIR (statmode))
+    str[0]='d';
+  if (S_ISREG (statmode))
+    str[0]='-';
+#ifdef S_ISFIFO
+  if (S_ISFIFO (statmode))
+    str[0]='p';
+#endif
+#ifdef S_ISLNK
+  if (S_ISLNK (statmode))
+    str[0]='l';
+#endif
+#ifdef S_ISSOCK
+  if (S_ISSOCK (statmode))
+    str[0]='s';
+#endif
+#ifdef S_ISMPC
+  if (S_ISMPC (statmode))
+    str[0]='m';
+#endif
+#ifdef S_ISNWK
+  if (S_ISNWK (statmode))
+    str[0]='n';
+#endif
+rwx ((statmode & 0700) << 0, &str[1]);
+rwx ((statmode & 0070) << 3, &str[4]);
+rwx ((statmode & 0007) << 6, &str[7]);
+setst (statmode, str);
+}
+
+
+static void filebrowser_properties_lcb(GtkWidget *widget, gpointer data) {
+	gchar *filename;
+	struct stat fileinfo;
+	struct passwd *uinfo;
+	struct group *ginfo;
+	filename = filebrowser_get_filename_from_row(main_v->filebrowser.file_list_selectedrow);
+	if (filename) {
+		gchar *label, *head;
+    
+    DEBUG_MSG("stat(2) start\n");
+    
+    if (stat(filename,&fileinfo) == 0) {
+      uinfo = getpwuid(fileinfo.st_uid);
+#ifndef WIN32
+      ginfo = getgrgid(fileinfo.st_gid);
+#endif
+	  filemode_to_string(fileinfo.st_mode);
+#ifndef WIN32
+	  label = g_strdup_printf(_("Size: %lu\nLast Modification: %sLast access: %sUser: %s\nGroup: %s\nMode: %s\n"),(long)fileinfo.st_size, 
+        ctime(&(fileinfo.st_mtime)), ctime(&(fileinfo.st_atime)),uinfo->pw_name, ginfo->gr_name,str);
+			head = g_strdup_printf(_("Properties of %s"),filename);
+#else
+	  label = g_strdup_printf(_("Size: %lu\nLast Modification: %sLast access: %sUser: %s\nMode: %s\n"),(long)fileinfo.st_size, 
+        ctime(&(fileinfo.st_mtime)), ctime(&(fileinfo.st_atime)),uinfo->pw_name, str);
+			head = g_strdup_printf(_("Properties of %s"),filename);
+
+#endif
+			error_dialog(head, label);
+			g_free(label);
+			g_free(head);
+    } else {
+      error_dialog(_("Error!"),_("Could not stat(2) file."));
+    }
+		g_free(filename);
+	}
+}
+
 static void file_list_rpopup_filter(GtkWidget *widget, gpointer filter_type) {
 	main_v->filebrowser.filter_type = (Tfilebrowser_filter) GPOINTER_TO_INT(filter_type);
 	populate_dir_file_list();
@@ -799,7 +1058,7 @@ static GtkWidget *file_list_rpopup_create_menu()
 	menu = gtk_menu_new();
 	
 	menu_item = gtk_menu_item_new_with_label(_("File new"));
-	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(file_new_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(create_file_lcb), NULL);
 	gtk_menu_append(GTK_MENU(menu), menu_item);
 	if (main_v->filebrowser.file_list_selectedrow != -1) {
 		/* a file is selected */
@@ -817,7 +1076,31 @@ static GtkWidget *file_list_rpopup_create_menu()
 		gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(filebrowser_file_delete_lcb), NULL);
 		gtk_menu_append(GTK_MENU(menu), menu_item);
 	}
-
+	menu_item = gtk_menu_item_new_with_label(_("Create directory"));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(create_directory_lcb), NULL);
+	gtk_menu_append(GTK_MENU(menu), menu_item);
+	if (main_v->filebrowser.file_list_selectedrow != -1) {
+		gchar *filename;
+		/* insert a separator */
+		menu_item = gtk_menu_item_new();
+		gtk_menu_append(GTK_MENU(menu), menu_item);
+		menu_item = gtk_menu_item_new_with_label(_("Properties"));
+		gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(filebrowser_properties_lcb), NULL);
+		gtk_menu_append(GTK_MENU(menu), menu_item);
+		
+		filename = filebrowser_get_filename_from_row(main_v->filebrowser.file_list_selectedrow);
+		if (identify_by_filename(filename) == TYPE_IMAGE) {
+#ifdef HAVE_IMAGE_LIBS
+			menu_item = gtk_menu_item_new_with_label(_("Preview image"));
+			gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(filebrowser_preview_image_lcb), NULL);
+			gtk_menu_append(GTK_MENU(menu), menu_item);
+#endif
+			menu_item = gtk_menu_item_new_with_label(_("Edit image"));
+			gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(filebrowser_edit_image_lcb), NULL);
+			gtk_menu_append(GTK_MENU(menu), menu_item);
+		}
+		g_free(filename);
+	}
 	/* insert a separator */
 	menu_item = gtk_menu_item_new();
 	gtk_menu_append(GTK_MENU(menu), menu_item);
@@ -831,6 +1114,7 @@ static GtkWidget *file_list_rpopup_create_menu()
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), fmenu);
 	
 	fmenu_item[0] = gtk_radio_menu_item_new_with_label(NULL, _("all files"));
+
 	if (main_v->filebrowser.filter_type == FILTER_ALL) {
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (fmenu_item[0]), TRUE);
  	}
@@ -971,14 +1255,14 @@ static void file_list_drag_data_get_lcb( GtkWidget *widget, GdkDragContext *cont
 	switch( info ) {
 		case TARGET_STRING:
 			DEBUG_MSG("file_list_drag_data_get_lcb, TARGET_STRING wanted\n");
-			filename = g_strconcat(main_v->filebrowser.curdir, "/", temp, NULL);
+			filename = g_strconcat(main_v->filebrowser.curdir, DIRSTR, temp, NULL);
 			break;
 		case TARGET_URI_LIST:
 			DEBUG_MSG("file_list_drag_data_get_lcb, TARGET_URI_LIST wanted\n");
-			filename = g_strconcat("file:", main_v->filebrowser.curdir, "/", temp, "\r\n", NULL);
+			filename = g_strconcat("file:", main_v->filebrowser.curdir, DIRSTR, temp, "\r\n", NULL);
 			break;
 		default:
-			filename = g_strconcat("file:", main_v->filebrowser.curdir, "/", temp, "\r\n", NULL);		
+			filename = g_strconcat("file:", main_v->filebrowser.curdir, DIRSTR, temp, "\r\n", NULL);		
 			break;
 		}
 	gtk_selection_data_set( selectionData, selectionData->target,8, filename, strlen(filename));
@@ -1096,12 +1380,25 @@ static GtkWidget *make_filelist() {
 	gtk_widget_show(vbox2);
 	gtk_notebook_append_page(GTK_NOTEBOOK(noteb), vbox2, gtk_label_new(_("Project")));
 
+#ifdef ATTRPAGE
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_widget_show(vbox2);
+	gtk_notebook_append_page(GTK_NOTEBOOK(noteb), vbox2, gtk_label_new(_("Attrs.")));
+	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
+				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_container_set_border_width (GTK_CONTAINER(scrolled_win), 5);
+	gtk_box_pack_start(GTK_BOX(vbox2), scrolled_win , TRUE, TRUE, 0);
+	gtk_widget_show (scrolled_win);
+	main_v->attrpage_window = scrolled_win;
+#endif
+
 	return noteb;
 }
 
 static void make_main_window(void)
 {
-	GtkWidget *vbox, *noteb, *hpane;
+	GtkWidget *vbox;
 
 	/* main window */
 	DEBUG_MSG("make_main_window, started\n");
@@ -1155,11 +1452,10 @@ static void make_main_window(void)
 		show_gui(&main_v->props.v_custom_tb, &hidewidgets.cust_handle_box, make_cust_menubar);
 		
 	DEBUG_MSG("make_main_window, creating lower region\n");
-	
-	hpane = gtk_hpaned_new();
-	gtk_paned_set_gutter_size(GTK_PANED(hpane), 8);
 
-	noteb = make_filelist();
+	main_v->middlebox = gtk_hbox_new(TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), main_v->middlebox, TRUE, TRUE, 0);
+	gtk_widget_show(main_v->middlebox);
 
 	/* notebook with the text widget in there */
 	main_v->notebook = gtk_notebook_new();
@@ -1175,11 +1471,12 @@ static void make_main_window(void)
 	gtk_notebook_set_page(GTK_NOTEBOOK(main_v->notebook), 0);
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(main_v->notebook), TRUE);
 	gtk_widget_show(main_v->notebook);
-
-	gtk_paned_add1(GTK_PANED(hpane), noteb);
-	gtk_paned_add2(GTK_PANED(hpane), main_v->notebook);
-	gtk_box_pack_start(GTK_BOX(vbox), hpane, TRUE, TRUE, 0);
-	gtk_widget_show(hpane);
+	
+	if (main_v->props.v_filebrowser) {
+		filebrowser_show(1);
+	} else {
+		filebrowser_hide(1);
+	}
 
 	/* status bar */
 	main_v->statusbar = gtk_statusbar_new();
@@ -1189,9 +1486,48 @@ static void make_main_window(void)
 	DEBUG_MSG("make_main_window, end\n");
 }
 
+void filebrowser_hide(gint first_time) {
+	if (!first_time) {
+		gtk_widget_ref(main_v->notebook);
+		gtk_container_remove(GTK_CONTAINER(main_v->filebrowser.hpane), main_v->notebook);
+		gtk_widget_destroy(main_v->filebrowser.hpane);
+	}
+	main_v->filebrowser.hpane = NULL;
+	DEBUG_MSG("filebrowser_hide, middlebox=%p, notebook=%p\n", main_v->middlebox, main_v->notebook);
+	gtk_box_pack_start(GTK_BOX(main_v->middlebox), main_v->notebook, TRUE, TRUE, 0);
+	if (!first_time) {
+		gtk_widget_unref(main_v->notebook);
+	}
+}
+
+void filebrowser_show(gint first_time) {
+	GtkWidget *noteb;
+
+	if (!first_time) {
+		gtk_widget_ref(main_v->notebook);
+		gtk_container_remove(GTK_CONTAINER(main_v->middlebox), main_v->notebook);
+	}
+
+	main_v->filebrowser.hpane = gtk_hpaned_new();
+	gtk_paned_set_gutter_size(GTK_PANED(main_v->filebrowser.hpane), 8);
+	
+	noteb = make_filelist();
+	
+	gtk_paned_add1(GTK_PANED(main_v->filebrowser.hpane), noteb);
+	gtk_paned_add2(GTK_PANED(main_v->filebrowser.hpane), main_v->notebook);
+	gtk_box_pack_start(GTK_BOX(main_v->middlebox), main_v->filebrowser.hpane, TRUE, TRUE, 0);
+
+	DEBUG_MSG("filebrowser_show, middlebox=%p, notebook=%p\n", main_v->middlebox, main_v->notebook);
+
+	gtk_widget_show(main_v->filebrowser.hpane);
+
+	if (!first_time) {
+		gtk_widget_unref(main_v->notebook);
+	}
+}
+
 void startup(void)
 {
-	GdkPixmap *icon = NULL;
 #ifndef NOSPLASH
 	GtkWidget *vbox, *splash_window;
 	GtkWidget *wpixmap;
@@ -1254,15 +1590,21 @@ void startup(void)
 	context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(main_v->statusbar), "Bluefish");
 	DEBUG_MSG("startup, main_window is ready\n");
 	/* add icon to the window when in minimized state */
-	icon = gdk_pixmap_create_from_xpm_d(main_v->main_window->window, NULL, NULL, (gchar **) bluefish_icon_xpm);
-	gdk_window_set_icon(main_v->main_window->window, NULL, icon, NULL);
+	{
+	GdkPixmap *icon = NULL;
+	GdkBitmap *imask = NULL;
+	icon = gdk_pixmap_create_from_xpm_d(main_v->main_window->window, &imask, NULL, (gchar **) bluefish_icon_xpm);
+	gdk_window_set_icon(main_v->main_window->window, NULL, icon, imask);
 	gdk_window_set_icon_name(main_v->main_window->window, CURRENT_VERSION_NAME);
 	DEBUG_MSG("startup, icon is there\n");
+	}
 	load_menu_shortcuts();	
 #ifndef NOSPLASH
 	/* kill the splash window when the main window is going up */
 	flush_queue();
+#ifndef WIN32
 	sleep(1);
+#endif
 	window_destroy(splash_window);
 #endif
 	gtk_widget_show(main_v->main_window);
