@@ -40,6 +40,7 @@
 void toggle_main_toolbar_cb(GtkWidget * w, gpointer data);
 void toggle_html_toolbar_cb(GtkWidget * w, gpointer data);
 void toggle_cust_menubar_cb(GtkWidget * w, gpointer data);
+void toggle_filebrowser_cb(GtkWidget * w, gpointer data);
 void file_and_dir_history_add(gchar *filename);
 void file_and_dir_history_remove(gchar *filename);
 void statusbar_message(gpointer message, gint time);
@@ -52,9 +53,12 @@ gboolean switch_to_document_by_filename(gchar *filename);
 void notebook_changed(void);
 void notebook_set_tab_pos(gchar * position);
 void filelist_set_size();
+void filebrowser_hide(gint first_time);
+void filebrowser_show(gint first_time);
 void startup(void);
 void show_about_window_cb(GtkWidget * w, gpointer data);
 void dndwin_cb (GtkWidget * widget, gpointer data);
+
 void populate_dir_file_list(void);
 void populate_dir_history(void);
 
