@@ -21,7 +21,6 @@
 #ifndef __MENU_H_
 #define __MENU_H_
 
-#include <locale.h>
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #include "intl.h"
@@ -33,8 +32,8 @@ void get_main_menu(GtkWidget * window, GtkWidget ** menubar);
 void make_cust_menubar(GtkWidget *cust_handle_box);
 
 
-extern void open_recent_list();
-extern void add_to_recent_list(gchar *filename);
+void open_recent_list();
+void add_to_recent_list(gchar *filename, gint closed_file);
 
 #ifdef ENABLE_NLS
 gchar *menu_translate(const gchar * path, gpointer data);
