@@ -35,8 +35,6 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 
-#include <locale.h>
-
 #include "bluefish.h"
 #include "interface.h"
 #include "network.h"
@@ -54,6 +52,7 @@ gint closed = 0;
 
 gint open_web(gchar * address)
 {
+#ifndef WIN32
 	gchar *server;
 	gchar *ip_address;
 	gchar *page_location;
@@ -162,7 +161,9 @@ gint open_web(gchar * address)
 			   since the loop will run on flush_queue() till there is
 			   some data on the socket again
 			 */
+#ifndef WIN32
 			sleep(1);
+#endif
 		} else if (received > 0) {
 			DEBUG_MSG("open_web, received=%d, >0\n", received);
 
@@ -230,9 +231,12 @@ gint open_web(gchar * address)
 
 	close(sockfd);
 	gtk_text_thaw(GTK_TEXT(main_v->current_document->textbox));
-	main_v->current_document->modified = 1;
+	doc_set_modified(main_v->current_document, 1);
 	DEBUG_MSG("open_web, document thawed, end\n");
 	return 0;
+#else
+	return 6;
+#endif
 }
 
 gint send_email_cb(gchar * address)
@@ -243,12 +247,15 @@ gint send_email_cb(gchar * address)
 
 gchar *resolve(gchar * address)
 {
-
+#ifndef WIN32
 	struct hostent *h;
 	if ((h = gethostbyname(address)) == NULL)
 		return "1";
 
 	return inet_ntoa(*(struct in_addr *) h->h_addr_list[0]);
+#else
+	return "1";
+#endif
 }
 
 gchar *extract_address(gchar * address)
