@@ -22,10 +22,10 @@
 
 #include <glib.h>
 
-gint open_web (gchar * address);
-gint send_email (gchar * address);
-gchar *resolve (gchar * address);
-gchar *extract_address (gchar * address);
-gchar *extract_filename (gchar * address, gchar * server);
+gint open_web(gchar * address);
+gint send_email_cb(gchar * address);
+gchar *resolve(gchar * address);
+gchar *extract_address(gchar * address);
+gchar *extract_filename(gchar * address, gchar * server);
 
-#endif /* __NETWORK_H__ */
+#endif							/* __NETWORK_H__ */
