@@ -20,8 +20,6 @@
  */
 #include "default_include.h"
 
-#include <locale.h>
-
 #include "../pixmaps/new.xpm"
 #include "../pixmaps/close.xpm"
 #include "../pixmaps/copy.xpm"
@@ -160,6 +158,7 @@
 #include "../pixmaps/var.xpm"
 #include "../pixmaps/javascript_mouseover.xpm"
 #include "../pixmaps/javascript_netscape.xpm"
+#include "../pixmaps/javascript_preload.xpm"
 
 
 GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background)
@@ -596,6 +595,9 @@ GtkWidget *new_pixmap(int type, GdkWindow * window, GdkColor * background)
 	case 501:
 		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, javascript_netscape_xpm);
 		break;
+	case 502:
+		pixm = gdk_pixmap_create_from_xpm_d(window, &mask, background, javascript_preload_xpm);
+		break;
 
 	}
 
