@@ -19,7 +19,6 @@
  */
 #include "default_include.h"
 
-#include <locale.h>
 #include <string.h>  /* memset() */
 
 #include "bluefish.h"
@@ -253,12 +252,17 @@ static gint test_modified_docs_in_project() {
 	while (tmpdoclist) {
 		tmpprolist = g_list_first(main_v->current_project.filenames);
 		while (tmpprolist) {
-			if (((Tdocument *)tmpdoclist->data)->filename && (strcmp(((Tdocument *)tmpdoclist->data)->filename, (gchar *) tmpprolist->data) == 0)) {
-				if (((Tdocument *)tmpdoclist->data)->modified) {
-					return 1;
-				}
-			}
+
+                     if  (((Tdocument*)tmpdoclist->data)->filename) { 
+                          if (strcmp(((Tdocument *)tmpdoclist->data)->filename, (gchar *) tmpprolist->data) == 0) {
+                               
+                               if (((Tdocument *)tmpdoclist->data)->modified) {
+                                  return 1;
+                             }
+                          }
+                     }
 			tmpprolist = g_list_next(tmpprolist);
+                
 		}
 		tmpdoclist = g_list_next(tmpdoclist);
 	}
@@ -296,6 +300,9 @@ static void project_empty(void)
 		}
 	
 		tmpprolist = g_list_first(main_v->current_project.filenames);
+                if (tmpdoc->filename == NULL) {
+                     tmpprolist = NULL;
+                }
 		while (tmpprolist && tmpdoclist) {
 			if (strcmp(tmpdoc->filename, (gchar *) tmpprolist->data) == 0) {
 				switch (retval) {
@@ -414,8 +421,8 @@ void project_open_cb(GtkWidget * widget, gpointer data)
 	g_free(tmpfilename);
 	DEBUG_MSG("project_open, main_v->current_project.filename=%p\n", main_v->current_project.filename);
 	tmpfilename = return_file(main_v->current_project.filename);
-	if (file_exists_and_readable(tmpfilename) == 1) {
 
+	if ((tmpfilename != NULL) && (file_exists_and_readable(tmpfilename) == 1)) {
 		/* close old project first */
 		project_close_cb(NULL, NULL);
 		if (main_v->current_project.filename != NULL) {
