@@ -22,8 +22,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include <locale.h>
-
 #include "bluefish.h"
 #include "bf_lib.h"
 #include "ref_dialog.h"
@@ -218,10 +216,10 @@ static void new_ref_dialog(RefClass * the_class, gchar * title)
 	gtk_table_set_col_spacing(GTK_TABLE(table), 1, 5);
 	gtk_widget_show(table);
 	gtk_container_add(GTK_CONTAINER(refdialog->dialog), table);
-	label = gtk_label_new("Group");
+	label = gtk_label_new(_("Group"));
 	gtk_widget_show(label);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
-	label = gtk_label_new("Function");
+	label = gtk_label_new(_("Function"));
 	gtk_widget_show(label);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
 	DEBUG_MSG("new_ref_dialog, euhhhh\n");
@@ -309,3 +307,76 @@ static void insert(GtkWidget * widget, RefDialog * data)
 	tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(data->functioncombo)->entry));
 	insert_dbl_text(tmp, "");
 }
+
+
+/* 
+ * get the php3 class
+ */
+static RefClass *php3_get_class()
+{
+	/* make sure it is properly initialised */
+	if (php3_ref == NULL) {
+		php3_ref = ref_init("/.bluefish/php3_functions");
+	}
+	return php3_ref;
+}
+
+/* 
+ * get the php4 class
+ */
+static RefClass *php4_get_class()
+{
+	/* make sure it is properly initialised */
+	if (php4_ref == NULL) {
+		php4_ref = ref_init("/.bluefish/php4_functions");
+	}
+	return php4_ref;
+}
+
+/*
+  Look for the description of a php(3|4) function
+*/
+static gchar *php_get_function_desc(RefClass *rfclass, gchar *function_name)
+{
+  GList *grplist, *funlist;
+  gchar fname[200];
+ 
+  fname[0] = ' ';
+  strcpy(fname + 1, function_name);
+  strcat(fname, "(");
+  if( rfclass ) {
+    for(grplist = rfclass->groupstructlist;
+	grplist;
+	grplist = g_list_next(grplist) ) {
+      for(funlist = ((RefGroup *)grplist->data)->functions;
+	  funlist;
+	  funlist = g_list_next(funlist) ) {
+	if( strstr(funlist->data, fname) != NULL )
+	  return funlist->data;
+      }
+    }
+  }
+  return NULL;
+}
+
+/*
+  Look for the description of a php3 function
+*/
+gchar *php3_get_function_desc(gchar *function_name)
+{
+  return php_get_function_desc(php3_get_class(), function_name);
+}
+
+/*
+  Look for the description of a php4 function
+*/
+gchar *php4_get_function_desc(gchar *function_name)
+{
+  return php_get_function_desc(php4_get_class(), function_name);
+}
+
+
+
+
+
+
