@@ -31,7 +31,13 @@
 #include "wml.h"
 #include "images.h"
 #include "document.h"
+
+#ifdef UNDO2
+#include "undo2.h" /* undo_cb */
+#else
 #include "undo.h"
+#endif /* UNDO2 */
+
 #include "highlight.h"
 
 /**************************************************************************/
