@@ -22,6 +22,6 @@
 #define __RPOPUP_H_
 
 
-gint rpopup_eventh(GtkWidget * widget, GdkEvent * event);
+void rpopup_eventh(GdkEventButton* bevent);
 
 #endif							/* __RPOPUP_H_ */
