@@ -34,7 +34,6 @@
  *            \           / /
  *             search_backend
  */
-
 /*****************************************************/
 #include "default_include.h"
 
@@ -42,11 +41,14 @@
 #include <string.h>
 #include <sys/types.h>  /* _before_ regex.h for freeBSD */
 #include <regex.h> 				/* regcomp() */
-#include <locale.h>	
-
 
 #include "bluefish.h"
 #include "bf_lib.h"
+
+#ifdef UNDO2
+#include "undo2.h" /* doc_unre_new_group */
+#endif
+
 #include "document.h"			/* doc_replace_text() */
 #include "interface.h"			/* g */
 #include "gtk_easy.h"         /* a lot of GUI functions */
@@ -62,19 +64,24 @@ typedef struct {
 	/* span of byte-data in buffer */
 	gint bstart;
 	gint bend;
+	
+	/* these data are only used (and alloc'ed) if want_submatches is set in the search backend,
+	they should be freed by the calling function! */
+	regmatch_t *pmatch;
+	gint nmatch;
 } Tsearch_result;
 
 /* Note:
     If Gtk's multibyte text support was enabled, a byte-offset
-  of a string can not be used as a character-offset of GtkText.
+    of a string can not be used as a character-offset of GtkText.
     The member "start" and "end" are character-offset.
     The member "bstart" and "bend" are byte-offset in the source
-  buffer.
+    buffer.
     These values are set in search_backend().
     In single-byte text mode,
         start == bstart, end == bend .
 
-   iwasa
+    iwasa
 */
 
 
@@ -147,16 +154,20 @@ static void reset_last_snr2(void) {
 
 /* for multibyte-text support.
    wchar_len() returns -1 if it failed to convert.
+	
+	src_len -1 means the complete string
 */
-static
 gint wchar_len(gchar *src, gint src_len)
 {
 	gchar *tmpbuf;
 	GdkWChar *tmpwbuf;
 	gint convlen = -1;
 
-	if (src_len == 0)
-		return 0;
+	if (src_len == -1) {
+		src_len = strlen(src);
+	}
+	if (src_len == 0) return 0;
+
 	tmpbuf = g_strndup(src, src_len);
 	if (tmpbuf) {
 		tmpwbuf = g_new(GdkWChar, src_len);
@@ -169,19 +180,20 @@ gint wchar_len(gchar *src, gint src_len)
 	return convlen;
 }
 
+/***********************************************************/
 
-Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens, gchar *buf, gboolean use_mb) {
+Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens, gchar *buf, gboolean use_mb, gboolean want_submatches) {
 
 	Tsearch_result returnvalue;
 	int (*f) ();
 	gint buflen, patlen, match, i;
-	regex_t reg_pat;
-	regmatch_t pmatch[1];
 
 	returnvalue.start = -1;
 	returnvalue.end = -1;
 	returnvalue.bstart = -1;
 	returnvalue.bend = -1;
+	returnvalue.pmatch = NULL;
+	returnvalue.nmatch = 0;
 	if ((!pattern) || (!buf)) {
 		DEBUG_MSG("search_backend, pattern or buf is NULL\n");
 		return returnvalue;
@@ -189,18 +201,55 @@ Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens,
 	
 	if (is_regex) {
 		/* regex part start */
+		regex_t reg_pat;
+		regmatch_t *pmatch;
+		gint nmatch, retval;
+
 		if (is_case_sens) {
-			regcomp(&reg_pat, pattern, REG_EXTENDED);
+			retval = regcomp(&reg_pat, pattern, REG_EXTENDED);
 		} else {
-			regcomp(&reg_pat, pattern, REG_EXTENDED | REG_ICASE);
+			retval = regcomp(&reg_pat, pattern, REG_EXTENDED | REG_ICASE);
 		}
-		if (!(regexec(&reg_pat, buf, 1, pmatch, 0) == 0)) {
+		DEBUG_MSG("search_backend, regcomp retval=%d\n", retval);
+		if (retval != 0) {
+#define ERRORSTR_SIZE 300
+			gchar errorstr[ERRORSTR_SIZE], *errorstr2;
+			
+			regerror(retval,  &reg_pat, errorstr, ERRORSTR_SIZE);
+			errorstr2 = g_strconcat(_("Regular expression error: "), errorstr, NULL);
+			error_dialog(_("Bluefish warning: regular expression error"), errorstr2);
+			g_free(errorstr2);
+			/* error compiling the pattern, returning the default result set,
+			which is the 'nothing found' set */
+			return returnvalue;
+		}
+		nmatch = (want_submatches) ? reg_pat.re_nsub+1 : 1;
+		DEBUG_MSG("search_backend, expr. contains %d sub patterns\n", reg_pat.re_nsub );
+		pmatch = g_malloc(nmatch*sizeof(regmatch_t));
+		retval = regexec(&reg_pat, buf, nmatch, pmatch, 0);
+		DEBUG_MSG("search_backend, regexec retval=%d\n", retval);
+		if (retval != 0) {
 			pmatch[0].rm_so = -1;
 			pmatch[0].rm_eo = -1;
-		} 
+		}
+#ifdef DEBUG
+		{	int i;
+			for (i=0;i<nmatch;i++) {
+				DEBUG_MSG("search_backend, sub pattern %d so=%d, eo=%d\n", i, pmatch[i].rm_so, pmatch[i].rm_eo);
+			}
+		}
+#endif
 		returnvalue.bstart = pmatch[0].rm_so;
 		returnvalue.bend = pmatch[0].rm_eo;
 		regfree(&reg_pat);
+		if (want_submatches) {
+			returnvalue.pmatch = pmatch;
+			returnvalue.nmatch = nmatch;
+			/* if want_submatches is set, pmatch should be 
+			free`ed by the calling function! */
+		} else {
+			g_free(pmatch);
+		}
 		/* regex part end */
 	} else {
 		/* non regex part start */
@@ -240,6 +289,7 @@ Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens,
 		returnvalue.start = returnvalue.bstart;
 		returnvalue.end = returnvalue.bend;
 	}
+	DEBUG_MSG("search_backend, returning result.start=%d, result.end=%d\n", returnvalue.start, returnvalue.end);
 	return returnvalue;
 }
 
@@ -248,14 +298,18 @@ Tsearch_result search_backend(gchar *pattern, gint is_regex, gint is_case_sens,
 Tsearch_result search_doc(Tdocument *document, gchar *pattern, gint is_regex, gint is_case_sens, gint startpos) {
 	gchar *fulltext;
 	Tsearch_result result;
-
-	DEBUG_MSG("search_doc, started\n");
+	
+	DEBUG_MSG("search_doc, started on document %p, startpos=%d\n", document, startpos);
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(document->textbox), startpos, -1);
-	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(document->textbox)->use_wchar);
+	DEBUG_MSG("search_doc, fulltext=%p, pattern=%p\n", fulltext, pattern);
+	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(document->textbox)->use_wchar, 0);
 	g_free(fulltext);
 	if (result.end > 0) {
-		last_snr2.result.start = result.start + startpos;
-		last_snr2.result.end = result.end + startpos;
+		DEBUG_MSG("search_doc, received a result (start=%d), adding startpos (%d) to it\n", result.start, startpos);
+		result.start += startpos;
+		result.end += startpos;
+		last_snr2.result.start = result.start;
+		last_snr2.result.end = result.end;
 		last_snr2.doc = document;
 	} else {
 		last_snr2.result.start = -1;
@@ -296,6 +350,7 @@ Tsearch_all_result search_all(gchar *pattern, gint is_regex, gint is_case_sens)
 	if (last_snr2.doc) {
 		tmplist = g_list_find(main_v->documentlist, last_snr2.doc);
 	} else {
+		last_snr2.result.end = 0;
 		tmplist = g_list_first(main_v->documentlist);
 	}
 	while (tmplist) {
@@ -304,30 +359,104 @@ Tsearch_all_result search_all(gchar *pattern, gint is_regex, gint is_case_sens)
 			result_all.start = result.start;
 			result_all.end = result.end;
 			result_all.doc = tmplist->data;
-			DEBUG_MSG("search_all, found!!!\n");
+			DEBUG_MSG("search_all, found!! start=%d, end=%d, doc=%p\n", result.start, result.end, tmplist->data);
 			return result_all;
-		} 
+		}
 		tmplist = g_list_next(tmplist);
+		if (tmplist) {
+			last_snr2.result.end = 0;
+		}
 	}
 	DEBUG_MSG("search_all, not found..\n");
 	return result_all;
 }
 
 /*****************************************************/
+/*
+this function will parse the replace string and substitute the \0, \1 etc. with 
+the subpattern matches from regexec()
+*/
+static gchar *reg_replace(gchar *replace_string, gint offset, Tsearch_result result, Tdocument *doc) {
+	gchar *tmp1, *newstring;
+	gchar *tmpstr1, *tmpstr2, *tmpstr3;
+	gboolean escaped=0;
+	guint from=0, to=0;
 
-Tsearch_result replace_backend(gchar *pattern, gint is_regex, gint is_case_sens, gchar *buf, gchar *replace_string, Tdocument *doc, gint offset, replacetypes replacetype, gint use_mb) {
+	newstring = g_strdup("");
+	tmp1 = replace_string;
+	while (*tmp1 != '\0') {
+		if (escaped) {
+			if (*tmp1 == '\\') {
+				to++;
+				tmpstr1 = newstring;
+				tmpstr2 = g_strndup(&replace_string[from], to-from);
+				newstring = g_strconcat(tmpstr1, tmpstr2, NULL);
+				g_free(tmpstr1);
+				g_free(tmpstr2);
+				from = ++to;
+				DEBUG_MSG("reg_replace, escaped backslash, newstring='%s'\n", newstring);			
+			} else if ((gint)*tmp1 >= 48 && (gint)*tmp1 <= 57) {
+				gint num = (gint)*tmp1 - 48;
+				tmpstr1 = newstring;
+				tmpstr2 = g_strndup(&replace_string[from], to-from);
+				DEBUG_MSG("reg_replace, from=%d, to=%d, tmpstr2='%s'\n", from, to, tmpstr2);
+				if (result.nmatch >= num+1) {
+					DEBUG_MSG("reg_replace, wanted: sub pattern %d, offset=%d, so=%d, eo=%d\n", num, offset, result.pmatch[num+1].rm_so, result.pmatch[num+1].rm_eo);
+					tmpstr3 = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), offset+result.pmatch[num+1].rm_so, offset+result.pmatch[num+1].rm_eo);
+					DEBUG_MSG("reg_replace, subpattern %d = '%s'\n", num, tmpstr3);
+				} else {
+					DEBUG_MSG("reg_replace, subpattern %d does not exist\n", num);
+					tmpstr3 = g_strdup("");
+				}
+				newstring = g_strconcat(tmpstr1, tmpstr2, tmpstr3, NULL);
+				g_free(tmpstr1);
+				g_free(tmpstr2);
+				g_free(tmpstr3);
+				to += 2;
+				from = to;
+				DEBUG_MSG("reg_replace, substituted subpattern, newstring='%s'\n", newstring);
+			} else {
+				to += 2;
+			}
+			escaped = 0;
+		} else {
+			if (*tmp1 == '\\') {
+				escaped = 1;
+			} else {
+				to++;
+			}
+		}
+		tmp1++;
+	}
+	tmpstr1 = newstring;
+	tmpstr2 = g_strndup(&replace_string[from], to-from);
+	newstring = g_strconcat(tmpstr1, tmpstr2, NULL);
+	g_free(tmpstr1);
+	g_free(tmpstr2);
+	DEBUG_MSG("reg_replace, end, newstring='%s'\n", newstring);
+	return newstring;
+}
+
+Tsearch_result replace_backend(gchar *pattern, gint is_regex, gint is_case_sens
+			, gchar *buf, gchar *replace_string, Tdocument *doc, gint offset, replacetypes replacetype
+			, gint use_mb, gint *replacelen) {
 /* the offset in this function is the difference between the buffer and the text widget because of previous replace 
 actions, so the first char in buf is actually number offset in the text widget */
-
+/* replacelen -1 means there is no replacelen known yet, so we have to calculate it */
 	Tsearch_result result;
 	gchar *tmpstr=NULL;
 	
-	result = search_backend(pattern, is_regex, is_case_sens, buf, use_mb);
+	result = search_backend(pattern, is_regex, is_case_sens, buf, use_mb, is_regex);
 	DEBUG_MSG("replace_backend, offset=%d, result.start=%d, result.end=%d\n", offset, result.start, result.end);
 	if (result.end > 0) {
 		switch (replacetype) {
 		case string:
-			tmpstr = g_strdup(replace_string);
+			if (is_regex) {
+				tmpstr = reg_replace(replace_string, offset, result, doc);
+			} else {
+				tmpstr = g_strdup(replace_string);
+			}
+			DEBUG_MSG("replace_backend, tmpstr='%s'\n", tmpstr);
 		break;
 		case upcase:
 			tmpstr = g_strndup(&buf[result.bstart], result.bend - result.bstart);
@@ -339,9 +468,20 @@ actions, so the first char in buf is actually number offset in the text widget *
 		break;
 		}
 		DEBUG_MSG("replace_backend, len=%d, offset=%d, start=%d, end=%d, document=%p\n", result.end - result.start, offset, result.start + offset, result.end + offset, doc);
-		doc_replace_text(tmpstr, result.start + offset, result.end + offset, doc);
+		doc_replace_text_backend(tmpstr, result.start + offset, result.end + offset, doc);
+		if (*replacelen == -1) {
+			if (GTK_TEXT(doc->textbox)->use_wchar) {
+				*replacelen = strlen(tmpstr);
+			} else {
+				*replacelen = wchar_len(tmpstr, -1);
+			}
+		}
 		g_free(tmpstr);
 	}
+	if (is_regex) {
+		g_free(result.pmatch);
+		result.pmatch = NULL;
+	}
 	return result;
 }
 
@@ -350,10 +490,15 @@ actions, so the first char in buf is actually number offset in the text widget *
 Tsearch_result replace_doc_once(gchar *pattern, gint is_regex, gint is_case_sens, gint startpos, gint endpos, gchar *replace_string, Tdocument *doc, replacetypes replacetype) {
 /* endpos -1 means do till end */
 	gchar *fulltext;
+	gint replacelen = 0; /* replacelen -1 means there is no replacelen known yet
+								, but we don't need one, so we set it to 0 */
 	Tsearch_result result;
 
+#ifdef UNDO2
+	doc_unre_new_group(doc);
+#endif
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), startpos, endpos);
-	result = replace_backend(pattern, is_regex, is_case_sens, fulltext, replace_string, doc, startpos, replacetype, GTK_TEXT(doc->textbox)->use_wchar);
+	result = replace_backend(pattern, is_regex, is_case_sens, fulltext, replace_string, doc, startpos, replacetype, GTK_TEXT(doc->textbox)->use_wchar, &replacelen);
 	if ( result.end > 0) {
 		last_snr2.result.start = result.start + startpos;
 		last_snr2.result.end = result.end + startpos;
@@ -364,6 +509,9 @@ Tsearch_result replace_doc_once(gchar *pattern, gint is_regex, gint is_case_sens
 		last_snr2.doc = doc;
 	}
 	g_free(fulltext);
+#ifdef UNDO2
+	doc_unre_new_group(doc);
+#endif
 	return result;
 }
  
@@ -376,21 +524,25 @@ void replace_doc_multiple(gchar *pattern, gint is_regex, gint is_case_sens, gint
 	Tsearch_result result;
 	gint in_buf_offset=0;
 	gint buf_text_offset=startpos;
-	gint fixedreplacelen=0;
+	gint replacelen=-1; /* replacelen -1 means there is no replacelen known yet */
 
-	if (replacetype == string) {
+#ifdef UNDO2
+	doc_unre_new_group(doc);
+#endif
+	DEBUG_MSG("replace_doc_multiple, startpos=%d, endpos=%d\n", startpos, endpos);
+	if (!is_regex || replacetype != string) {
+		/* the replace string has a fixed length if it is not regex, or it is not type string */
 		if (GTK_TEXT(doc->textbox)->use_wchar) {
-			fixedreplacelen = wchar_len(replace_string, strlen(replace_string));
+			replacelen = wchar_len(replace_string, -1);
 		} else {
-			fixedreplacelen = strlen(replace_string);
+			replacelen = strlen(replace_string);
 		}
 	}
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), startpos, endpos);
-	result = replace_backend(pattern, is_regex, is_case_sens, fulltext, replace_string, doc, buf_text_offset, replacetype, GTK_TEXT(doc->textbox)->use_wchar);
-
+	result = replace_backend(pattern, is_regex, is_case_sens, fulltext, replace_string, doc, buf_text_offset, replacetype, GTK_TEXT(doc->textbox)->use_wchar, &replacelen);
 	while (result.end > 0) {
 		if (replacetype == string) {
-			buf_text_offset += fixedreplacelen - (result.end - result.start);
+			buf_text_offset += replacelen - (result.end - result.start);
 		}
 		if (last_snr2.overlapping_search) {
 			if (GTK_TEXT(doc->textbox)->use_wchar) {
@@ -402,7 +554,7 @@ void replace_doc_multiple(gchar *pattern, gint is_regex, gint is_case_sens, gint
 				}
 				g_free(onechar);
 			} else {
-				in_buf_offset += result.bstart + 1;;
+				in_buf_offset += result.bstart + 1;
 			}
 			buf_text_offset += result.start + 1;
 		} else {
@@ -410,11 +562,17 @@ void replace_doc_multiple(gchar *pattern, gint is_regex, gint is_case_sens, gint
 			buf_text_offset += result.end;
 		}
 		DEBUG_MSG("replace_doc_multiple, after first search, buf_text_offset=%d, in_buf_offset=%d\n", buf_text_offset, in_buf_offset);
-
-		result = replace_backend(pattern, is_regex, is_case_sens, &fulltext[in_buf_offset], replace_string, doc, buf_text_offset, replacetype, GTK_TEXT(doc->textbox)->use_wchar);
+		if (is_regex && replacetype == string) {
+			/* all regex replaces can have different replace lengths, so they have to be re-calculated */
+			replacelen = -1;
+		}
+		result = replace_backend(pattern, is_regex, is_case_sens, &fulltext[in_buf_offset], replace_string, doc, buf_text_offset, replacetype, GTK_TEXT(doc->textbox)->use_wchar, &replacelen);
 
 		DEBUG_MSG("replace_doc_multiple, 1- buf_text_offset=%d, in_buf_offset=%d, result.start=%d, result.end=%d\n", buf_text_offset, in_buf_offset, result.start, result.end);
 	}
+#ifdef UNDO2
+	doc_unre_new_group(doc);
+#endif
 	last_snr2.result.start = -1;
 	last_snr2.result.end = -1;
 	last_snr2.doc = doc;
@@ -494,7 +652,7 @@ gint replace_prompt_doc(gchar *pattern, gint is_regex, gint is_case_sens, gint s
 	Tsearch_result result;
 
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), startpos, endpos);
-	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(doc->textbox)->use_wchar);
+	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(doc->textbox)->use_wchar, 0);
 	last_snr2.doc = doc;
 	g_free(fulltext);
 	if (result.end > 0) {
@@ -592,7 +750,7 @@ void snr2_run(void) {
 			doc_show_result(result_all.doc, result_all.start, result_all.end);
 		} else {
 			result = search_doc(main_v->current_document, last_snr2.pattern, last_snr2.is_regex, last_snr2.is_case_sens, startpos);
-			doc_show_result(main_v->current_document, result.start + startpos, result.end + startpos);	
+			doc_show_result(main_v->current_document, result.start, result.end);	
 		}
 	}
 }
@@ -615,7 +773,14 @@ static void snr2dialog_replacetype_toggled(GtkWidget *widget, Tsnr2_win *data) {
 /*****************************************************/
 
 static void snr2dialog_ok_lcb(GtkWidget *widget, Tsnr2_win *data) {
-
+	if (last_snr2.pattern) {
+		g_free(last_snr2.pattern);
+		last_snr2.pattern = NULL;
+	}
+	if (last_snr2.replace_string) {
+		g_free(last_snr2.replace_string);
+		last_snr2.replace_string = NULL;
+	}
 	last_snr2.pattern = gtk_editable_get_chars(GTK_EDITABLE(data->pattern), 0, -1);
 	last_snr2.region_from_beginning = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->region_from_beginning));
 	last_snr2.region_from_cursor = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->region_from_cursor));
@@ -641,6 +806,57 @@ static void snr2dialog_ok_lcb(GtkWidget *widget, Tsnr2_win *data) {
 	snr2_run();
 }
 
+/* void snr2_run_extern_replace
+arguments are translated to last_snr2 like this:
+pattern = patten
+region,  0 = region_from_beginning, 
+			1 = region_from_cursor, 
+			2 = region_selection,
+			3 = region_all_open_files
+is_regex = is_regex
+is_case_sens = is_case_sens
+overlapping_search is off
+replace = 1
+replace_string = replace_string
+prompt_before_replacing = off
+replace_once = off
+replacetype_string = 1
+replacetype_upcase = 0
+replacetype_lowcase = 0
+*/
+void snr2_run_extern_replace(gchar *pattern, gint region,
+							gint is_regex, gint is_case_sens, gchar *replace_string) {
+	if (!pattern || !replace_string || !strlen(pattern)) {
+		DEBUG_MSG("snr2_run_extern, returning, non-valid arguments\n");
+		return;
+	}
+	if (last_snr2.pattern) {
+		g_free(last_snr2.pattern);
+		last_snr2.pattern = NULL;
+	}
+	if (last_snr2.replace_string) {
+		g_free(last_snr2.replace_string);
+		last_snr2.replace_string = NULL;
+	}
+	last_snr2.pattern = g_strdup(pattern);
+	last_snr2.region_from_beginning = region == 0 ? 1: 0;
+	last_snr2.region_from_cursor = region == 1 ? 1: 0;
+	last_snr2.region_selection = region == 2 ? 1: 0;
+	last_snr2.region_all_open_files = region == 3 ? 1: 0;
+	last_snr2.is_regex = is_regex;
+ 	last_snr2.is_case_sens = is_case_sens;
+ 	last_snr2.overlapping_search = 0;
+	last_snr2.replace = 1;
+	last_snr2.replace_string = g_strdup(replace_string);
+ 	last_snr2.prompt_before_replacing = 0;
+ 	last_snr2.replace_once = 0;
+	last_snr2.replacetype_string = 1;
+	last_snr2.replacetype_upcase = 0;
+	last_snr2.replacetype_lowcase = 0;
+
+	snr2_run();
+}
+
 static void snr2dialog(gint is_replace, gint is_new_search) {
 
 	Tsnr2_win *snr2win;
@@ -725,6 +941,8 @@ static void snr2dialog(gint is_replace, gint is_new_search) {
 	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
 	gtk_window_set_default(GTK_WINDOW(snr2win->window), but);
 	gtk_box_pack_start(GTK_BOX(hbox), bf_stock_cancel_button(snr2dialog_cancel_lcb, snr2win), TRUE, TRUE, 0);
+	
+	gtk_widget_grab_focus(snr2win->pattern);
 	gtk_widget_show_all(snr2win->window);
 	
 }
@@ -835,7 +1053,7 @@ void update_filenames_in_file(Tdocument *doc, gchar *oldfilename, gchar *newfile
 	}
 
 	fulltext = gtk_editable_get_chars(GTK_EDITABLE(doc->textbox), 0, -1);
-	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(doc->textbox)->use_wchar);
+	result = search_backend(pattern, is_regex, is_case_sens, fulltext, GTK_TEXT(doc->textbox)->use_wchar, 0);
 	while (result.end > 0) {
 		if (doc_has_newfilename) {
 			changelen = do_filename_curfile_replace(fulltext, result, cur_offset, olddirname, newfilename, changelen, doc);
@@ -843,7 +1061,7 @@ void update_filenames_in_file(Tdocument *doc, gchar *oldfilename, gchar *newfile
 			changelen = do_filename_otherfile_replace(fulltext, result, cur_offset, oldfilename, newfilename, changelen, doc);
 		}
 		cur_offset += result.bstart +1;
-		result = search_backend(pattern, is_regex, is_case_sens, &fulltext[cur_offset], GTK_TEXT(doc->textbox)->use_wchar);
+		result = search_backend(pattern, is_regex, is_case_sens, &fulltext[cur_offset], GTK_TEXT(doc->textbox)->use_wchar, 0);
 	}
 	g_free(fulltext);
 	if (doc_has_newfilename) {
