@@ -29,6 +29,8 @@ void paste_cb(GtkWidget * w, gpointer data);
 void cut_cb(GtkWidget * w, gpointer data);
 void sel_all_cb(GtkWidget * w, gpointer data);
 
+void snr2_run_extern_replace(gchar *pattern, gint region,
+							gint is_regex, gint is_case_sens, gchar *replace_string);
 void search_cb(GtkWidget *widget, gpointer data);
 void new_search_cb(GtkWidget *widget, gpointer data);
 void replace_again_cb(GtkWidget *widget, gpointer data);
@@ -36,4 +38,6 @@ void search_again_cb(GtkWidget *widget, gpointer data);
 void replace_cb(GtkWidget *widget, gpointer data);
 void new_replace_cb(GtkWidget *widget, gpointer data);
 void update_filenames_in_file(Tdocument *doc, gchar *oldfilename, gchar *newfilename, gint doc_has_newfilename);
+
+gint wchar_len(gchar *src, gint src_len);
 #endif							/* __SNR_H_ */
