@@ -2,6 +2,7 @@
  * spell.c - Check Spelling
  *
  * Copyright (C)2000 Pablo De Napoli (for this module)
+ * Minor updates Copyright (C)2001 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +23,10 @@
 */
 
 /* For the select function */
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+#ifdef WITH_SPC
 #define _GNU_SOURCE
 
 
@@ -49,10 +54,6 @@
 /* This is needed when the stdin is redirected */
 #define MY_DEBUG_MSG g_printerr
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
-
 #include "intl.h"
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -63,6 +64,7 @@
 #include "gtk_easy.h"
 #include "char_table.h"
 #include "interface.h"
+#include "document.h"
 
 #include <unistd.h>
 #include <fcntl.h>
@@ -101,7 +103,7 @@ enum {
 
 static FILE *in, *out;  /* streams to communicate with ispell */
 pid_t isp_pid = -1; /* pid for the `ispell' process */
-
+GdkColor misspelled_color; /* tb */
 static int isp_fd;
 
 /* void sigchldhandler(int sig); */
@@ -171,7 +173,7 @@ static gchar get_next_char()
            current_position = old_position;
            c = old_c;
            /* FIXME: mark for gettext / beter dialog ? */
-           g_printerr("Invalid especial character! %s\n",token);
+           g_printerr(_("Invalid especial character! %s\n"),token);
          }
     }
   }
@@ -334,14 +336,14 @@ void create_ispell_pipe()
 
 		argc = 0;
 		argv[argc++] = main_v->props.cfg_spc_cline ;
-		argv[argc++] = g_strdup("-a"); // "Pipe" mode
+		argv[argc++] = g_strdup("-a"); /* "Pipe" mode */
 
 		if ((g_strcasecmp(main_v -> props.cfg_spc_lang, "default")!=0) &&
           (*(main_v -> props.cfg_spc_lang)!='\0'))
       {
          if (main_v ->props.cfg_spc_lang != NULL)
          {
-          argv[argc++] = g_strdup("-d"); // Dictionary file
+          argv[argc++] = g_strdup("-d"); /* Dictionary file */
 			 argv[argc++] = main_v -> props.cfg_spc_lang;
          };
 		}
@@ -562,24 +564,42 @@ static isp_result * ispell_check_word (gchar *word)
 
 static inline void select_word (Tword* word)
 {
-  gtk_editable_select_region (GTK_EDITABLE(CURRENT_DOCUMENT_TEXT),word->begin,word->end);
-
+	doc_unbind_signals(main_v->current_document);
+  	gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT), word->begin); 
+	gtk_text_forward_delete(GTK_TEXT(CURRENT_DOCUMENT_TEXT), word->end - word->begin);/* tb */
+	gtk_text_insert(GTK_TEXT(CURRENT_DOCUMENT_TEXT), 0, &misspelled_color, 0,word->text, -1);/* tb */
+	gtk_text_thaw(GTK_TEXT(CURRENT_DOCUMENT_TEXT));/* tb */			
+	/* gtk_editable_select_region (GTK_EDITABLE(CURRENT_DOCUMENT_TEXT),word->begin,word->end); *//* tb */
+	doc_bind_signals(main_v->current_document);
 }
 
 /*for gtk_text_insert*/
 #define ALL_THE_STRING -1
-
 static void replace_word_with (Tword *word,gchar* replace_with)
 {
-  DEBUG_MSG("Deleting word \"%s\" \n",word->text);
-  gtk_editable_delete_selection(GTK_EDITABLE(CURRENT_DOCUMENT_TEXT));
+	replace_text(replace_with, word->begin, word->end);
+/*
+  DEBUG_MSG("Deleting word \"%s\" \n",word->text);  
+  gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT), word->begin);
+  gtk_text_forward_delete(GTK_TEXT(CURRENT_DOCUMENT_TEXT), word->end - word->begin);
   DEBUG_MSG("inserting replace text =\"%s\" \n",replace_with);
   gtk_editable_insert_text(GTK_EDITABLE(CURRENT_DOCUMENT_TEXT), replace_with,strlen(replace_with),&(word->begin));
-  /* DEBUG_MSG("setting new point\n");
-  gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT), word->begin + strlen(replace_with)-1); */
+*/
   /* FIXME: convert iso characters to html ? */
 }
 
+/*
+ *change slected word color back to original color
+ */
+static void change_word_color(Tword * word) {
+	doc_unbind_signals(main_v->current_document);
+	gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT), word->begin);
+	gtk_text_forward_delete(GTK_TEXT(CURRENT_DOCUMENT_TEXT), word->end - word->begin);
+	DEBUG_MSG("changing text color");
+	gtk_editable_insert_text(GTK_EDITABLE(CURRENT_DOCUMENT_TEXT), word->text, strlen(word->text),&(word->begin));
+	doc_bind_signals(main_v->current_document);
+}
+
 static void correct_word(Tword* word)
 /* ask the user how to correct a word, and do it */
 {
@@ -593,17 +613,21 @@ static void correct_word(Tword* word)
      switch (spc_message)
      {
      case SPC_INSERT:      ispell_insert_word(word->text);
+	     		        change_word_color(word);
                            word_corrected = 1;
                            break;
      case SPC_ACCEPT:      ispell_accept_word(word->text);
+	                   change_word_color(word);
                            word_corrected = 1;
 			                  break;
-     case SPC_IGNORE:      word_corrected = 1;
+     case SPC_IGNORE:      change_word_color(word);
+	     		        word_corrected = 1;
                            break;
      case SPC_REPLACE:     replace_word_with (word,gtk_entry_get_text(GTK_ENTRY(spc_gui.replace_entry)));
                            word_corrected = 1;
                            break;
      case SPC_CLOSE:       DEBUG_MSG("close message\n");
+	     		        change_word_color(word);
                            word_corrected = 1; /* to exit the while loop */
                            break;
 
@@ -665,17 +689,8 @@ static void check_word (Tword* word)
 } /* end check_word  */
 
 
-inline static void update_progress_bar(void)
-{
- gfloat percentage;
- gfloat current_position;
- gfloat text_length;
- current_position = (gfloat) gtk_text_get_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
- text_length = (gfloat) gtk_text_get_length(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
- /* percentage is betweeen 0 and 1 */
- percentage = current_position / text_length;
- DEBUG_MSG ("percentage done = %f\n",percentage);
- gtk_progress_bar_update(GTK_PROGRESS_BAR(spc_gui.progress_bar),percentage);
+inline static void update_progress_bar(void) {
+	gtk_progress_bar_update(GTK_PROGRESS_BAR(spc_gui.progress_bar),(gfloat) gtk_text_get_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT))/ (gfloat) gtk_text_get_length(GTK_TEXT(CURRENT_DOCUMENT_TEXT)));
 };
 
 void ispell_terminate()
@@ -686,24 +701,53 @@ void ispell_terminate()
 	fflush(out);
 	fclose(out);
    kill(isp_pid,SIGTERM);
-   isp_pid = -1; /* ispell is not running.FIXME: Can we check this ? */
+	if (waitpid(isp_pid, (int *)0, WNOHANG) != 0) {
+	   isp_pid = -1; /* ispell is not running.FIXME: Can we check this ? */
+	}
  }
 }
 
 /* Perform an ispell session */
-gboolean run_spell_checker (void)
+void run_spell_checker (void)
 {
 	Tword *word;
-   unsigned int word_count = 0;
-   gtk_text_freeze(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
-     /* create ispell process */
-	
-    create_ispell_pipe();
-    gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT),0);
-    end_of_text = 0; /* needed for get_next_word to work properly */
 
-    if (isp_pid == -1) {
-		                 error_dialog(_("Bluefish Spell Checker Error"),
+   misspelled_color.red = 0xffff; /* tb */
+   misspelled_color.green = 0; /* tb */
+   misspelled_color.blue = 0; /* tb */
+   if (!gdk_color_alloc(main_v->colormap, &misspelled_color)) { /* tb */
+	g_error("couldn't allocate required colors"); /* tb */
+   } /* tb */
+   gtk_text_freeze(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
+     /* create ispell process */	
+	ispell_terminate();
+	create_ispell_pipe();
+	gtk_text_set_point(GTK_TEXT(CURRENT_DOCUMENT_TEXT),0);
+	end_of_text = 0; /* needed for get_next_word to work properly */
+
+	if (isp_pid != -1) {
+       /* Put ispell in terse mode to improve speed */
+	    ispell_terse_mode();
+	    do
+       {
+        word = get_next_word();
+        if (!is_empty_word(word))
+        {
+         spc_message = SPC_NONE;
+         check_word(word);
+         free_word (word);
+         update_progress_bar();
+         if (spc_message==SPC_CLOSE)
+                      {
+                      DEBUG_MSG("Close message received\n");
+                      break;
+                      };
+         };
+       } while (!end_of_text);
+      }
+      else
+            {
+            error_dialog(_("Bluefish Spell Checker Error"),
 			              _("\n\n"
 			              "The ispell-process has died for some reason. *One* possible reason\n"
 			              "could be that you do not have a dictionary file\n"
@@ -711,42 +755,16 @@ gboolean run_spell_checker (void)
 			              "Check /usr/lib/ispell or set another\n"
 			              "dictionary in the Spellchecker Options menu."));
 		                 fclose(out);
-		                 return TRUE;
-                     }; /*end if*/
-
-   gtk_grab_add(spc_gui.window); /* This window becames modal */
-
-	/* Put ispell in terse mode to improve speed */
-	ispell_terse_mode();
-	do
-   {
-      word = get_next_word();
-      if (!is_empty_word(word))
-      {
-      word_count++;
-      spc_message = SPC_NONE;
-      check_word(word);
-      free_word (word);
-      update_progress_bar();
-      if (spc_message==SPC_CLOSE)
-                      {
-                      DEBUG_MSG("Close message received\n");
-                      break;
-                      };
-     };
-   } while (!end_of_text);
-   DEBUG_MSG("Spell checking done\n");
-  gtk_text_thaw(GTK_TEXT(CURRENT_DOCUMENT_TEXT));
-  ispell_terminate();
-  DEBUG_MSG("Removing grab\n");
-  gtk_grab_remove(spc_gui.window);
-  DEBUG_MSG("Removing status \n");
-  statusbar_remove(GINT_TO_POINTER(spc_gui.status_bar_count));
-  DEBUG_MSG("Destroying window\n");
-  gtk_widget_destroy(spc_gui.window);
-
-  return (spc_message != SPC_CLOSE);
-} /* end run_spell_checker */
+            }; /*end if*/
+      DEBUG_MSG("Removing grab\n");
+      gtk_grab_remove(spc_gui.window);
+      DEBUG_MSG("Removing status \n");
+      statusbar_remove(GINT_TO_POINTER(spc_gui.status_bar_count));
+      DEBUG_MSG("Destroying window\n");
+      gtk_widget_destroy(spc_gui.window);
+		ispell_terminate();
+  } /* end run_spell_checker */
 
 
 
+#endif /* WITH_SPC */
