@@ -29,7 +29,7 @@
 
 #define CURRENT_DOCUMENT_TEXT main_v->current_document->textbox
 
-gboolean run_spell_checker(void);
+void run_spell_checker(void);
 
 void ispell_terminate(void);
 
