@@ -21,7 +21,6 @@
 
 
 #include <stdio.h>
-#include <locale.h>
 #include <string.h>  	/* strspn() */
 #include "bluefish.h"
 #include "bf_lib.h"
@@ -139,7 +138,11 @@ static void estrl_down_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
 
 static void estrl_add_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
 
+#ifdef __GNUC__
 	gchar *tmplistentry[(estrl->num_columns ? estrl->num_columns  : 1)];
+#else
+	gchar *tmplistentry[65535];
+#endif
 	guint i;
 
 	for (i = 0; i < (estrl->num_columns ? estrl->num_columns  : 1); i++) {
@@ -306,6 +309,27 @@ gint count_array(gchar **array) {
 	return count;
 }
 
+GList *duplicate_stringlist(GList *list, gint dup_data) {
+	GList *retlist=NULL;
+	if (list) {
+		GList *tmplist;
+		tmplist = g_list_first(list);
+		while (tmplist) {
+			if (tmplist->data) {
+				gchar *data;
+				if (dup_data) {
+					data = g_strdup((gchar *)tmplist->data);
+				} else {
+					data = (gchar *)tmplist->data;
+				}
+				retlist = g_list_append(retlist, data);
+			}
+			tmplist = g_list_next(tmplist);
+		}
+	}
+	return retlist;
+}
+
 gint free_stringlist(GList * which_list)
 {
 	GList *tmplist;
