@@ -19,8 +19,6 @@
  */
 #include "default_include.h"
 
-#include <locale.h>
-
 #include <string.h>
 #include <dirent.h>
 #include <sys/types.h>
@@ -28,7 +26,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-
 #include "bluefish.h"
 #include "pixmaps.h"
 #include "toolbars.h"
@@ -46,12 +43,17 @@
 #include "wizards.h"
 #include "ref_dialog.h"
 #include "configure.h"
+
+#ifdef UNDO2
+#include "undo2.h"
+#else
 #include "undo.h"
+#endif /* UNDO2 */
+
 #include "snr2.h"
 #include "gtk_easy.h"
 
 /* define WITH_SPC */
-#include "config.h"
 #include "spell_gui.h"
 
 typedef struct {
@@ -96,11 +98,11 @@ void make_main_toolbar(GtkWidget *handle_box)
 	gtk_container_add(GTK_CONTAINER(handle_box), main_widget);
 	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("New"), "",
 							new_pixmap(000, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(file_new_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Open"), "",
+	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Open..."), "",
 							new_pixmap(001, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(file_open_cb), NULL);
 	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Save"), "",
 							new_pixmap(002, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(file_save_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Save As"), "",
+	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Save As..."), "",
 							new_pixmap(003, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(file_save_as_cb), NULL);
 	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Close"), "",
 							new_pixmap(004, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(file_close_cb), NULL);
@@ -111,24 +113,24 @@ void make_main_toolbar(GtkWidget *handle_box)
 							new_pixmap(006, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(cut_cb), NULL);
 	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Paste"), "",
 							new_pixmap(007, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(paste_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Search"), "",
+	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Search..."), "",
 							new_pixmap(010, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(search_cb), NULL);
 	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL,
-							_("Search and Replace"), "", new_pixmap(011,
+							_("Search and Replace..."), "", new_pixmap(011,
 																	main_v->main_window->window,
 																	NULL), GTK_SIGNAL_FUNC(replace_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(main_widget));
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Undo"), "",
+	main_v->toolb.undo = gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Undo"), "",
 							new_pixmap(012, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(undo_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Redo"), "",
+	main_v->toolb.redo = gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Redo"), "",
 							new_pixmap(013, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(redo_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(main_widget));
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Configure"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Configure..."),
 							"", new_pixmap(014, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(configure_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Print"), "",
+	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Print..."), "",
 							new_pixmap(015, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(file_print_cb), NULL);
 #ifdef WITH_SPC	
-	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Spellcheck"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(main_widget), NULL, _("Spellcheck..."),
 							"", new_pixmap(016, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(spell_check_cb), NULL);
 #endif	
 	gtk_toolbar_append_space(GTK_TOOLBAR(main_widget));
@@ -263,9 +265,9 @@ void make_html_toolbar(GtkWidget *html_handle_box)
 							"", new_pixmap(112, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(quickrule_cb), NULL);
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Image..."),
 							"", new_pixmap(113, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(image_insert_dialog_cb), NULL);
-#if HAVE_LIBGDK_IMLIB
+#ifdef HAVE_IMAGE_LIBS
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL,
-							"Thumbnail...",
+							_("Thumbnail..."),
 							"", new_pixmap(125, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(image_thumbnail_dialog_cb), NULL);
 #endif
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
@@ -303,7 +305,7 @@ void make_html_toolbar(GtkWidget *html_handle_box)
 																   NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 13);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 
-	/* headings dropdown menu */
+	/* context formatting dropdown menu */
 	sub_menu = gtk_menu_new();
 	/*abbr*/
 	menu_item = gtk_menu_item_new();
@@ -332,9 +334,15 @@ void make_html_toolbar(GtkWidget *html_handle_box)
 	/*dfn*/
 	menu_item = gtk_menu_item_new();
 	gtk_container_add(GTK_CONTAINER(menu_item), 
-				new_pixmap(422, main_v->main_window->window, NULL));
+				new_pixmap(421, main_v->main_window->window, NULL));
 	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 48);
 	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
+	/*kbd*/
+	menu_item = gtk_menu_item_new();
+	gtk_container_add(GTK_CONTAINER(menu_item), 
+				new_pixmap(422, main_v->main_window->window, NULL));
+	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 51);
+	gtk_menu_append(GTK_MENU(sub_menu), menu_item);
 	/*samp*/
 	menu_item = gtk_menu_item_new();
 	gtk_container_add(GTK_CONTAINER(menu_item), 
@@ -531,7 +539,7 @@ void make_html_toolbar(GtkWidget *html_handle_box)
 		gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Style"), "", new_pixmap(184,
 			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 42);
-		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Link to stylesheet"), "", new_pixmap(189,
+		gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Link to stylesheet..."), "", new_pixmap(189,
 			main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(link_cb), NULL);
 
 		flush_toolbar(_(" CSS "), html_toolbar, html_widget);
@@ -543,13 +551,14 @@ void make_html_toolbar(GtkWidget *html_handle_box)
 	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Javascript for Mouseover Effects"),
 							"", new_pixmap(500, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(mouseover_script_cb), NULL);
-	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Javascript to work around Resize Bug in Netscape 4"),
 							"", new_pixmap(501, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(ns47_bugfix_script_cb), NULL);
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Javascript preload images"),
+							"", new_pixmap(502, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(imagepreload_script_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Empty javascript block"),
 							"", new_pixmap(188, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_html_cb), (gpointer *) 44);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Link to javascript"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Link to javascript..."),
 							"", new_pixmap(187, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(script_cb), NULL);
 	flush_toolbar(_(" Javascript "), html_toolbar, html_widget);
 
@@ -557,17 +566,17 @@ void make_html_toolbar(GtkWidget *html_handle_box)
 	html_toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Standard WML Document"),
 							"", new_pixmap(400, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_wml_cb), (gpointer *) 6);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Card Element"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Card Element..."),
 							"", new_pixmap(402, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(carddialog_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Postfield Element"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Postfield Element..."),
 							"", new_pixmap(408, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(postfielddialog_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Go Element"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Go Element..."),
 							"", new_pixmap(410, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(godialog_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Do Element"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Do Element..."),
 							"", new_pixmap(414, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(dodialog_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Anchor Element"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Anchor Element..."),
 							"", new_pixmap(415, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(anchordialog_cb), NULL);
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Access Element"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Access Element..."),
 							"", new_pixmap(416, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(accessdialog_cb), NULL);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Bold"),
@@ -589,7 +598,7 @@ void make_html_toolbar(GtkWidget *html_handle_box)
 	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("No Operation"),
 							"", new_pixmap(413, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(general_wml_cb), (gpointer *) 9);
 	gtk_toolbar_append_space(GTK_TOOLBAR(html_toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Setvar"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(html_toolbar), NULL, _("Set variable.."),
 							"", new_pixmap(409, main_v->main_window->window, NULL), GTK_SIGNAL_FUNC(vardialog_cb), NULL);
 
 	flush_toolbar(_(" WML "), html_toolbar, html_widget);
