@@ -22,8 +22,6 @@
 #include "default_include.h"
 #include <string.h>
 
-#include <locale.h>
-
 #include "bluefish.h"
 #include "undo.h"
 #include "document.h"
@@ -259,7 +257,8 @@ static void undo_redo_common(gboolean which)
 	/* update the doc info label */
 	if ((d->modified != undo->changed) || which == 1) {
 		/* redo is always changed */
-		d->modified = which ? 1 : undo->changed;
+		doc_set_modified(d, which ? 1 : undo->changed);
+		/* removed in favour of doc_set_modified() d->modified = which ? 1 : undo->changed; */
 	}
 	DEBUG_MSG("undo_redo_common, d->modified=%d\n", d->modified);
 	/* now re-establish the signals */
