@@ -21,12 +21,8 @@
 #ifndef __WIZARDS_H_
 #define __WIZARDS_H_
 
-/* Need for some types used in head_cber file */
-#include <glib.h>
-#include <gtk/gtkwidget.h>
+void tablewizard(GtkWidget *widget, gpointer data);
+void framewizard(GtkWidget * widget, gpointer data);
 
-void quickstart(GtkWidget * widget, gpointer data);
-void tablewizard(void);
-void framewizard(void);
 
 #endif							/* __WIZARDS_H_ */
