@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
- * wml.h - menu/toolbar callback prototypes
+ * wml.h - WML support dialogs
  *
- * Copyright (C) 1998,2000 Olivier Sessink, Chris Mazuc, Roland Steinbach
+ * Copyright (C) 1999,2000 Olivier Sessink, Roland Steinbach
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,4 +30,4 @@ void dodialog_cb(GtkWidget * widget, gpointer data);
 void anchordialog_cb(GtkWidget * widget, gpointer data);
 void accessdialog_cb(GtkWidget * widget, gpointer data);
 
-#endif							/* __HTML_H_ */
+#endif							/* __WML_H_ */
